<?php



namespace WpieApp\Core\Helpers;

defined( 'WPIE__FILE__' ) || exit;


trait Validators {

        
        protected static $ksesAllowedTags = [];

        public static function isNumeric( $value = null ) {
                return is_float( $value ) || is_int( $value ) || (is_string( $value ) && preg_match( '#^[+-]?([0-9]++\.?[0-9]*|\.[0-9]+)$#D', $value ));
        }

        
        public static function isCallable( $var = "" ) {
                return Helper::isNonEmptyStr( $var ) && is_callable( $var, true );
        }

        
        public static function isEmpty( $var = null ) {

                if ( is_null( $var ) ) {
                        return true;
                }

                $type = \gettype( $var );

                if ( in_array( $type, [ "boolean", "integer", "double" ] ) ) {
                        return false;
                }

                if ( $type === "string" ) {
                        return !Helper::isNonEmptyStr( $var );
                }

                return empty( $var );
        }

        
        public static function isScalar( $var ) {
                return is_scalar( $var ) && !Helper::isEmpty( $var );
        }

        
        public static function isEmail( $email ) {

                // First, check if the email address is in a valid format
                if ( !Helper::isNonEmptyStr( $email ) || !Helper::contains( $email, '@' ) ) {
                        return false;
                }
                return filter_var( $email, FILTER_VALIDATE_EMAIL );
        }

        
        public static function isUrl( $value ) {

                $alpha = "a-z\x80-\xFF";

                $pattern = "/(^
				https?://(
					(([-_0-9$alpha]+\\.)*                       # subdomain
						[0-9$alpha]([-0-9$alpha]{0,61}[0-9$alpha])?\\.)?  # domain
						[$alpha]([-0-9$alpha]{0,17}[$alpha])?   # top domain
					|\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}  # IPv4
					|\\[[0-9a-f:]{3,39}\\]                      # IPv6
				)(:\\d{1,5})?                                   # port
				(/\\S*)?                                        # path
				(\\?\\S*)?                                      # query
				(\\#\\S*)?                                      # fragment
			$)Dix";

                return preg_match( $pattern, $value ) ? true : false;
        }

        
        public static function isUri( $value = "" ) {
                return ( bool ) preg_match( '#^[a-z\d+\.-]+:\S+$#Di', $value );
        }

        
        public static function removeScheme( $uri = "" ) {
                return Helper::isNonEmptyStr( $uri ) ? preg_replace( "#^[^:/.]*[:/]+#i", "", trim( $uri ) ) : "";
        }

        
        public static function ksesAllowedTags( $tags = [] ) {

                self::setKsesAllowedTags();

                return Helper::isNonEmptyArray( $tags ) ? \array_replace_recursive( $tags, self::$ksesAllowedTags ) : self::$ksesAllowedTags;
        }

        
        public static function setKsesAllowedTags() {

                if ( Helper::isNonEmptyArray( self::$ksesAllowedTags ) ) {
                        return;
                }

                // Get default allowed tags
                $allowedTags = \wp_kses_allowed_html( 'post' );

                self::$ksesAllowedTags = [
                        'a'      => isset( $allowedTags[ 'a' ] ) ? $allowedTags[ 'a' ] : [],
                        'img'    => isset( $allowedTags[ 'img' ] ) ? $allowedTags[ 'img' ] : [],
                        'strong' => isset( $allowedTags[ 'strong' ] ) ? $allowedTags[ 'strong' ] : [],
                ];
        }
}
