<?php



namespace WpieApp\Core\Office;

use WpieApp\Core\Helpers\Helper;
use WpieApp\Core\Office\Converter\Excel\Excel;
use WpieApp\Core\Office\Converter\Csv2Json;
use WpieApp\Core\Office\Converter\Csv2Xml;

defined( 'WPIE__FILE__' ) || exit;


class Converter {

        
        public static function Convert( $file = "", $format = "", $dest = "", $start = 0 ) {

                if ( !Helper::isNonEmptyStr( $file ) || !Helper::isNonEmptyStr( $format ) ) {
                        return new \WP_Error( 'wpieError', __( 'File path is not valid', 'wp-import-export' ) );
                }

                $format = Helper::isNonEmptyStr( $format ) ? Helper::sanitizeKey( $format ) : "";

                if ( !Helper::isNonEmptyStr( $format ) || !in_array( $format, [ "xls", "xlsx", "ods", "json", "csv", "xml" ] ) ) {
                        return new \WP_Error( 'wpieError', __( 'Invalid File Format', 'wp-import-export' ) );
                }

                if ( !is_readable( $file ) ) {
                        return new \WP_Error( 'wpieError', __( 'File is not readable', 'wp-import-export' ) );
                }

                $from = Helper::getFileExt( $file );

                if ( $format === $from ) {
                        return true;
                }

                $dir = pathinfo( $file, PATHINFO_DIRNAME );

                $start = Helper::toInt( $start );

                if ( !Helper::isNonEmptyStr( $dest ) ) {

                        $newName = \wp_unique_filename( $dir, Helper::sanitizeFilename( pathinfo( $file, PATHINFO_FILENAME ), $format ) );

                        $dest = ($dir . DIRECTORY_SEPARATOR . $newName);
                }

                $process = null;

                switch ( $from ) {
                        case "csv":
                                $process = self::fromCsv( $file, $format, $dest, $start );
                                break;
                        case "json":
                                //  $process = self::fromJson( $file, $format );
                                break;
                        case "xls":
                        case "xlsx":
                        case "ods":
                                //    $process = self::fromExcel( $file, $format );
                                break;
                }
                return $process;
        }

        
        public static function fromCsv( $file = "", $format = "", $dest = "", $start = 0 ) {

                if ( !Helper::isNonEmptyStr( $file ) ) {
                        return new \WP_Error( 'wpieError', __( 'File path is not valid', 'wp-import-export' ) );
                }

                if ( !is_readable( $file ) ) {
                        return new \WP_Error( 'wpieError', __( 'File is not readable', 'wp-import-export' ) );
                }

                $format = Helper::isNonEmptyStr( $format ) ? Helper::sanitizeKey( $format ) : "";

                if ( !Helper::isNonEmptyStr( $format ) || !in_array( $format, [ "xls", "xlsx", "ods", "json", "xml" ] ) ) {
                        return new \WP_Error( 'wpieError', __( 'Invalid File Format', 'wp-import-export' ) );
                }

                $newFile = null;

                switch ( $format ) {
                        case "xls":
                        case "xlsx":
                        case "ods":
                                $newFile = Excel::convert( $file, $dest );
                                break;
                        case "json":
                                $newFile = Csv2Json::convert( $file, $dest, $start );
                                break;
                        case "xml":
                                $newFile = Csv2Xml::convert( $file, $dest, $start );
                                break;
                }

                return $newFile;
        }

        
//        public static function fromJson( $file = "", $format = "" ) {
//
//
//                if ( !is_readable( $file ) ) {
//                        return new \WP_Error( 'wpieError', __( 'File is not readable', 'wp-import-export' ) );
//                }
//
//                $format = Helper::isNonEmptyStr( $format ) ? Helper::sanitizeKey( $format ) : "";
//
//                if ( !Helper::isNonEmptyStr( $format ) || !in_array( $format, [ "xml" ] ) ) {
//                        return new \WP_Error( 'wpieError', __( 'Invalid File Format', 'wp-import-export' ) );
//                }
//
//                $newFile = null;
//
//                switch ( $format ) {
//
//                        case "xml":
//                                $newFile = Json2Xml::convert( $file );
//                                break;
//                }
//
//                return $newFile;
//        }
//
//        
//        public static function fromExcel( $file = "", $format = "" ) {
//
//                if ( !is_readable( $file ) ) {
//                        return new \WP_Error( 'wpieError', __( 'File is not readable', 'wp-import-export' ) );
//                }
//
//                $format = Helper::isNonEmptyStr( $format ) ? Helper::sanitizeKey( $format ) : "";
//
//                if ( !Helper::isNonEmptyStr( $format ) || !in_array( $format, [ "xml" ] ) ) {
//                        return new \WP_Error( 'wpieError', __( 'Invalid File Format', 'wp-import-export' ) );
//                }
//
//                $newFile = null;
//
//                switch ( $format ) {
//
//                        case "xml":
//                                $newFile = Excel2Xml::convert( $file );
//                                break;
//                }
//
//                return $newFile;
//        }
}
