<?php


namespace WpieApp\Core\Page\Blocks\Templates\Mapping;

use WpieApp\Core\Helpers\Helper;
use WpieApp\Core\Page\Blocks\Fields;
use WpieApp\Core\Page\Blocks\Fields\Alert;
use WpieApp\Core\Page\Blocks\Blocks;
use WpieApp\Core\Page\Blocks\Templates\Standard\Manager as Standard;

defined( 'WPIE__FILE__' ) || exit;


class Manager {

        
        public static function render( $data = [] ) {

                if ( !Helper::isNonEmptyArray( $data ) ) {
                        return;
                }
                $sectionClass = isset( $data[ "class" ] ) ? Helper::toString( $data[ "class" ] ) : "";
                $contentType  = isset( $data[ "contentType" ] ) ? $data[ "contentType" ] : "Posts";
                $groupClass   = isset( $data[ "groupClass" ] ) ? Helper::toString( $data[ "groupClass" ] ) : "";
                $notes        = isset( $data[ "notes" ] ) ? $data[ "notes" ] : [];
                $blocks       = isset( $data[ "setting-blocks" ] ) ? $data[ "setting-blocks" ] : [];
                ?>
                <div class="wpie-section mt-3 <?php echo esc_attr( $sectionClass . " " . $groupClass ); ?>">
                        <?php Alert::addNotes( $notes ); ?>
                        <div  class="row m-auto me-3 wpie-section-container bg-white gx-5">
                                <div class="col-12 my-5 border-bottom pb-4">
                                        <div class="text-center"><h2><?php esc_html_e( 'Field Mapping', 'wp-import-export' ); ?></h2></div>
                                </div>
                                <div class="col-md-4 d-none">
                                        <div class="m-auto text-center">
                                                <div class="mb-3 bse-text-orange bse-fw-500 bse-fs-18"><?php esc_html_e( 'Auto Detect', 'wp-import-export' ); ?></div>
                                        </div>
                                        <div class="border p-4 rounded-1 mb-3 bg-primary bg-opacity-10">
                                                <?php
                                                Fields::addDescription( __( 'Auto Detect settings', 'wp-import-export' ) );

                                                Fields::addSingleField( [
                                                        "type"       => "button",
                                                        "class"      => "wpie-auto-detect-settings btn-primary",
                                                        "icon-class" => "fa-solid fa-gears",
                                                        "label"      => __( 'Auto Detect', 'wp-import-export' )
                                                ] );
                                                ?>
                                        </div>
                                </div>
                                <div class="col-md-6">
                                        <div class="m-auto text-center">
                                                <div class="mb-3 bse-text-orange bse-fw-500 bse-fs-18"><?php esc_html_e( 'ADD / Update Items', 'wp-import-export' ); ?></div>
                                        </div>
                                        <div class="border p-4 rounded-1 mb-3 bg-primary bg-opacity-10">
                                                <?php
                                                Fields::addDescription( __( 'ADD / Update Items', 'wp-import-export' ) );

                                                Fields::addSingleField( [
                                                        "type"      => "checkbox",
                                                        "class"     => "wpie-handle-items",
                                                        "name"      => "handle-items",
                                                        "is-switch" => true,
                                                        "default"   => [ "add", "update" ],
                                                        "choices"   => [
                                                                "add"    => __( 'Add New Items', 'wp-import-export' ),
                                                                "update" => __( 'Update Existing Items', 'wp-import-export' ), ]
                                                ] );
                                                ?>
                                        </div>
                                </div>
                                <div class="col-md-6">
                                        <div class="m-auto text-center">
                                                <div class="mb-3 bse-text-orange bse-fw-500 bse-fs-18"><?php esc_html_e( 'Settings', 'wp-import-export' ); ?></div>
                                        </div>
                                        <div class="border p-4 rounded-1 mb-3 bg-primary bg-opacity-10">
                                                <?php
                                                Fields::addDescription( __( 'Load / Update / Save settings', 'wp-import-export' ) );

                                                Fields::addSingleField( [
                                                        "type"       => "button",
                                                        "class"      => "wpie-manage-settings btn-primary",
                                                        "icon-class" => "fa-solid fa-gears",
                                                        "label"      => __( 'Manage Settings', 'wp-import-export' )
                                                ] );
                                                ?>
                                        </div>
                                </div>
                                <div class="col-12">
                                        <div class="m-auto text-center border-bottom border-primary border-2 mt-5 mb-3">
                                                <div class="mb-3 text-primary bse-fw-600 bse-fs-24"><?php esc_html_e( 'Available Fields', 'wp-import-export' ); ?></div>
                                        </div>
                                </div>
                                <div class="col-12">
                                        <?php
                                        Alert::addNotes( [
                                                [
                                                        "type"       => "success",
                                                        "icon-class" => "fa-regular fa-circle-check",
                                                        "text"       => __( 'Feel free to leave any data unchanged. the plugin will use the default data provided by WordPress', 'wp-import-export' )
                                                ],
                                                [
                                                        "type"       => "success",
                                                        "icon-class" => "fa-regular fa-circle-check",
                                                        "text"       => __( 'Certain fields are automatically detected and mapped by the plugin. You can turn off these fields if you do not wish to update them', 'wp-import-export' )
                                                ],
                                                [
                                                        "type"       => "danger",
                                                        "icon-class" => "fa-solid fa-triangle-exclamation",
                                                        "text"       => __( 'Only fields that are enabled will be added or updated', 'wp-import-export' )
                                                ],
                                                [
                                                        "type"       => "info",
                                                        "icon-class" => "fa-solid fa-circle-info",
                                                        "text"       => __( 'To set the data, click the Edit icon', 'wp-import-export' )
                                                ],
                                                [
                                                        "type"       => "warning",
                                                        "icon-class" => "fa-solid fa-star",
                                                        "text"       => __( "Starred fields are important for creating new items, but you can leave them blank if you don't have data", 'wp-import-export' )
                                                ],
                                                [
                                                        "type"       => "info",
                                                        "icon-class" => "fa-solid fa-circle-info ",
                                                        "class"      => "mb-0 wpie-drag-and-drop-mapping-alert mt-3 me-3",
                                                        "text"       => sprintf(
                                                                /* translators: Link to the field mapping documentation. */
                                                                __(
                                                                        '<a href="%s" target="_blank" class="wpie-info-alert-link">Click here</a> to learn the step-by-step guide on mapping fields.',
                                                                        'wp-import-export'
                                                                ),
                                                                \esc_url( WPIE_DOC_URL . 'field-mapping' )
                                                        )
                                                ]
                                        ] );
                                        ?>

                                </div>
                                <div class="col-12"><div class="row wpie-available-fields"></div></div>

                        </div>

                </div>

                <?php
                self::settingModal( $blocks );
        }

        
        private static function settingModal( $section = [] ) {

                if ( !Helper::isNonEmptyArray( $section ) ) {
                        return;
                }

                $uid = Helper::randStr( "wpie-block-" );
                ?>

                <div class="modal fade <?php echo esc_html( $uid ); ?> wpie-settings-modal-container d-none">
                        <div class="modal-dialog modal-fullscreen">
                                <div class="modal-content">
                                        <div class="modal-header">
                                                <h5 class="modal-title" ><?php esc_html_e( 'Load / Update / Save Settings', 'wp-import-export' ); ?></h5>
                                                <div class="align-right ms-auto">
                                                        <button type="button" class="btn btn-secondary wpie-btn-cancel me-2" data-bs-dismiss="modal" aria-label="Close"><i class="bse-fs-14 me-1 fa-solid fa-close"></i><?php esc_html_e( 'Close', 'wp-import-export' ); ?></button>
                                                </div>
                                        </div>
                                        <div class="modal-body bg-light pt-0">
                                                <div class="row">
                                                        <div class="col-12">
                                                                <?php
                                                                Alert::render( [
                                                                        "type"       => "info",
                                                                        "icon-class" => "fa-solid fa-circle-info ",
                                                                        "class"      => "mb-0 wpie-settings-alert mt-3 me-3",
                                                                        "text"       => __( 'Load / Update / Save Settings and Load another import settings', 'wp-import-export' )
                                                                ] );

                                                                Standard::render( $section );
                                                                ?>
                                                        </div>

                                                </div>
                                        </div>
                                </div>
                        </div>
                </div>
                <?php
        }
}
