<?php


namespace WpieApp\Core\Page\Blocks\Templates\Standard;

use WpieApp\Core\Helpers\Helper;
use WpieApp\Core\Page\Blocks\Fields;
use WpieApp\Core\Page\Blocks\Fields\Alert;

defined( 'WPIE__FILE__' ) || exit;


class Manager {

        
        public static function render( $data = [] ) {

                if ( !Helper::isNonEmptyArray( $data ) ) {
                        return;
                }

                $blocks = isset( $data[ "blocks" ] ) ? $data[ "blocks" ] : "";

                if ( !Helper::isNonEmptyArray( $blocks ) ) {
                        return;
                }

                $blocks = Helper::sortBy( $blocks );

                $sectionClass      = isset( $data[ "class" ] ) ? Helper::toString( $data[ "class" ] ) : "";
                $groupClass        = isset( $data[ "groupClass" ] ) ? Helper::toString( $data[ "groupClass" ] ) : "";
                $groupName         = isset( $data[ "group-name" ] ) ? Helper::toString( $data[ "group-name" ] ) : "";
                $menuWrapperClass  = isset( $data[ "menuWrapperClass" ] ) ? Helper::toString( $data[ "menuWrapperClass" ] ) : "";
                $blockWrapperClass = isset( $data[ "blockWrapperClass" ] ) ? Helper::toString( $data[ "blockWrapperClass" ] ) : "col-12 col-lg-9 col-md-8 col-sm-8 shadow-sm bg-white";
                $notes             = isset( $data[ "notes" ] ) ? $data[ "notes" ] : [];
                ?>
                <div class="wpie-section mt-3 <?php echo esc_attr( $sectionClass . " " . $groupClass ); ?>">
                        <?php Alert::addNotes( $notes ); ?>
                        <div  class="row m-auto p-0 me-3 wpie-section-container">
                                <div class="col-12 col-lg-3 col-md-4 col-sm-4 wpie-menu-items <?php echo esc_attr( $menuWrapperClass ); ?> ">
                                        <?php
                                        self::addMenu( [ "title" => "&nbsp;", "desc" => "&nbsp;", "menuClass" => "invisible" ] );
                                        foreach ( $blocks as $key => $menu ) {
                                                if ( !Helper::isNonEmptyArray( $menu ) ) {
                                                        continue;
                                                }
                                                $blocks[ $key ][ 'slug' ] = $menu[ 'slug' ]           = Helper::randStr( "wpie-container-" );
                                                self::addMenu( $menu );
                                        }
                                        ?>

                                </div>
                                <div class="wpie-item-container <?php echo esc_attr( $blockWrapperClass ); ?>">

                                        <?php
                                        foreach ( $blocks as $block ) {
                                                
                                                if ( !Helper::isNonEmptyArray( $block ) ) {
                                                        continue;
                                                }
                                                $title           = isset( $block[ "title" ] ) ? $block[ "title" ] : "";
                                                $isActive        = isset( $block[ "isActive" ] ) && \absint( $block[ "isActive" ] ) === 1 ? "isActive" : "";
                                                $class           = isset( $block[ "class" ] ) ? $block[ "class" ] : "";
                                                $slug            = isset( $block[ "slug" ] ) ? $block[ "slug" ] : "";
                                                $notes           = isset( $block[ "notes" ] ) && Helper::isNonEmptyArray( $block[ "notes" ] ) ? $block[ "notes" ] : [];
                                                $fields          = isset( $block[ "fields" ] ) ? $block[ "fields" ] : "";
                                                $backdrop        = isset( $block[ "backdrop" ] ) && $block[ "backdrop" ] === true ? true : false;
                                                $proBackdrop     = isset( $block[ "pro-backdrop" ] ) && $block[ "pro-backdrop" ] === true ? true : false;
                                                $backdropOptions = isset( $block[ "backdropOptions" ] ) ? $block[ "backdropOptions" ] : [];
                                                ?>
                                                <div class="row p-4 bg-white wpie-section-data-container position-relative <?php echo esc_attr( $class . " " . $isActive . " " . $slug ); ?>">
                                                        <?php
                                                        if ( $proBackdrop ) {
                                                                self::addProBackdrop();
                                                        } elseif ( $backdrop ) {
                                                                self::addBackdrop( $backdropOptions );
                                                        }
                                                        Alert::addNotes( $notes );
                                                        ?>

                                                        <div class="col-12 wpie-section-header border-bottom mb-5 p-0"><h4 class="bse-fw-500 p-0 m-auto mt-2"><?php echo esc_html( $title ); ?></h4></div>
                                                        <div class="col-12 m-0 p-0">
                                                                <div class="row g-4">
                                                                        <?php Fields::render( $fields, [ 'group-name' => $groupName ] ); ?>
                                                                </div>
                                                        </div>
                                                </div>
                                                <?php
                                        }
                                        ?>
                                </div>
                        </div>
                </div>
                <?php
        }

        
        public static function addMenu( $data = [] ) {

                $title     = isset( $data[ "title" ] ) ? $data[ "title" ] : "";
                $desc      = isset( $data[ "desc" ] ) ? $data[ "desc" ] : "";
                $slug      = isset( $data[ "slug" ] ) ? $data[ "slug" ] : "";
                $menuClass = isset( $data[ "menuClass" ] ) ? $data[ "menuClass" ] : "";
                $isActive  = isset( $data[ "isActive" ] ) && \absint( $data[ "isActive" ] ) === 1 ? "isActive" : "";
                ?>
                <div class="row wpie-menu-item p-3 <?php echo esc_attr( $menuClass . " " . $isActive ); ?>" container="<?php echo esc_attr( $slug ); ?>">
                        <div class="col-12 wpie-menu-item-title bse-fs-16 bse-fw-500"><?php echo esc_html( $title ); ?></div>
                        <div class="col-12 wpie-menu-item-description mt-1 bse-fs-14"><?php echo esc_html( $desc ); ?></div>
                </div>
                <?php
        }

        
        public static function addBackdrop( $data = [] ) {

                if ( !Helper::isNonEmptyArray( $data ) ) {
                        return;
                }

                $title   = isset( $data[ 'title' ] ) ? $data[ 'title' ] : "";
                $desc    = isset( $data[ 'desc' ] ) ? $data[ 'desc' ] : "";
                $icon    = isset( $data[ 'icon' ] ) ? $data[ 'icon' ] : 'fa-solid fa-lock';
                $buttons = isset( $data[ 'buttons' ] ) ? $data[ 'buttons' ] : [];
                ?>
                <div class="wpie-backdrop position-absolute top-0 bottom-0 start-0 end-0 bg-white bg-opacity-75 z-2">
                        <div class="container p-5">
                                <div class="row text-center border rounded-1 bg-white shadow-lg p-4">                                        
                                        <div class="col-12 mb-3">
                                                <i class="<?php echo esc_attr( $icon ); ?> bse-fs-24 p-3 rounded-circle border border-danger text-danger border-2 "></i>
                                        </div>                                        
                                        <div class="col-12 mb-3 bse-fs-18 text-danger bse-fw-600">
                                                <?php echo esc_html( $title ); ?>
                                        </div>
                                        <div class="col-12 mb-5 ">
                                                <?php echo esc_html( $desc ); ?>
                                        </div>
                                        <div class="col-auto align-self-center m-auto">

                                                <?php
                                                if ( Helper::isNonEmptyArray( $buttons ) ) {
                                                        foreach ( $buttons as $btn ) {
                                                                $btnLabel = isset( $btn[ 'label' ] ) ? $btn[ 'label' ] : "";
                                                                $btnIcon  = isset( $btn[ 'icon' ] ) ? $btn[ 'icon' ] : "";
                                                                $btnClass = isset( $btn[ 'class' ] ) ? $btn[ 'class' ] : "";
                                                                $btnUrl   = isset( $btn[ 'url' ] ) ? $btn[ 'url' ] : "#";
                                                                ?>
                                                                <a href="<?php echo esc_attr( $btnUrl ); ?>" target="_blank" class="btn btn-lg mx-3 <?php echo esc_attr( $btnClass ); ?>"><i class="<?php echo esc_attr( $btnIcon ); ?> me-2"></i></i><?php echo esc_html( $btnLabel ); ?></a>
                                                                <?php
                                                        }
                                                }
                                                ?>

                                        </div>                                       
                                </div>

                        </div>
                </div>


                <?php
        }

        
        public static function addProBackdrop() {
                ?>
                <div class="wpie-pro-backdrop position-absolute top-0 bottom-0 start-0 end-0 bg-white bg-opacity-75 z-2">
                        <div class="container p-5">
                                <div class="row text-center p-4">                                        
                                        <div class="col-12 mb-3">
                                                <i class="fa-solid fa-lock bse-fs-24 p-3 rounded-circle border-2 border border-danger text-danger border-2 bg-white" ></i>
                                        </div>                                        
                                        <div class="col-auto m-auto mb-3 bse-fs-18 text-danger bse-fw-600 bg-white"><?php esc_html_e( 'Premium Feature', 'wp-import-export' ); ?></div>
                                </div>

                        </div>
                </div>


                <?php
        }
}
