<?php



namespace WpieApp\Core\Utils\Archive;

use WP_Error;
use Exception;
use WpieApp\Core\Helpers\Helper;
use WpieApp\Core\Admin\AutoLoader;
use wapmorgan\UnifiedArchive\UnifiedArchive;
use wapmorgan\UnifiedArchive\Formats;

defined( 'WPIE__FILE__' ) || exit;


class ArchiveHandler extends Base {

        
        public static function canOpen( $format ) {

                AutoLoader::loadArchive();

                try {
                        $canOpen = Formats::canOpen( $format );
                } catch ( Exception $e ) {
                        $canOpen = false;
                }

                return $canOpen;
        }

        
        public static function create( $archiveName, $fileOrFiles ) {

                AutoLoader::loadArchive();

                if ( method_exists( UnifiedArchive::class, 'create' ) ) {
                        UnifiedArchive::create( $fileOrFiles, $archiveName );
                } else {
                        UnifiedArchive::archiveFiles( $fileOrFiles, $archiveName );
                }
        }

        
        public static function open( $file = "", $password = null ) {

                $isReadable = Helper::isReadable( $file );

                if ( is_wp_error( $isReadable ) ) {
                        return $isReadable;
                }

                AutoLoader::loadArchive();

                if ( !UnifiedArchive::canOpen( $file ) ) {
                        /* translators: %s: File path */
                        return new WP_Error( 'wpieError', sprintf( __( "Fail to open Archive %s", 'wp-import-export' ), $file ) );
                }

                $password = Helper::isNonEmptyStr( $password ) ? $password : "";

                try {
                        $archive = UnifiedArchive::open( $file, $password );
                } catch ( Exception $e ) {
                        return new WP_Error( 'wpieError', $e->getMessage() );
                }
                if ( is_null( $archive ) ) {
                        /* translators: %s: File path */
                        return new WP_Error( 'wpieError', sprintf( __( "Fail to open Archive %s", 'wp-import-export' ), $file ) );
                }

                return $archive;
        }

        
        public static function getFileList( $file = "", $options = [] ) {

                $archive = self::open( $file );

                if ( \is_wp_error( $archive ) ) {
                        return $archive;
                }

                $files = [];

                $allowedExt = isset( $options[ 'allowedExt' ] ) && Helper::isNonEmptyArray( $options[ 'allowedExt' ] ) ? $options[ 'allowedExt' ] : [];

                foreach ( $archive->getFileNames() as $filename ) {

                        $ext = Helper::getFileExt( $filename );

                        if ( !Helper::isNonEmptyArray( $allowedExt ) || !in_array( $ext, $allowedExt ) ) {
                                continue;
                        }

                        $files[] = $filename;
                }
                if ( Helper::isNonEmptyArray( $files ) ) {
                        return $files;
                }

                return new WP_Error( 'wpieError', __( "Archive doesn't have valid file to import", 'wp-import-export' ) );
        }

        
        public static function extract( $file = "", $filename = "", $options = [] ) {

                $archive = self::open( $file );

                if ( is_wp_error( $archive ) ) {
                        return $archive;
                }

                if ( !$archive->hasFile( $filename ) ) {
                        return new WP_Error( 'wpieError', __( "File Not Found In Archive", 'wp-import-export' ) );
                }

                $outputFolder = isset( $options[ 'outputFolder' ] ) && Helper::isNonEmptyStr( $options[ 'outputFolder' ] ) ? $options[ 'outputFolder' ] : '';

                //Extract File
                try {
                        $name = $filename;

                        $archive->extractFiles( $outputFolder, $name );
                } catch ( Exception $e ) {

                        return new WP_Error( 'wpieError', $e->getMessage() );
                }

                //Check if Extract in sub Foders
                $extractFile = Helper::pathJoin( $outputFolder, $filename );
                $filePath    = Helper::pathJoin( $outputFolder, basename( $filename ) );
                if ( $filePath !== $extractFile ) {

                        Helper::getFilesystem()->move( $extractFile, $filePath );

                        self::removeExtraFolders( $filename, $outputFolder );
                }

                return $filePath;
        }
}
