<?php



namespace WpieApp\Extensions\Export\WooCommerce\Controller\Items\Order\Standard\DataCenters;

use WC_Data_Store;
use WpieApp\Modules\Export\Helpers\Helper;
use WpieApp\Extensions\Export\WooCommerce\Controller\Items\Order\Standard\DataCenter;

defined( 'WPIE__FILE__' ) || exit;


class Download extends Base {

        
        private static $items = [];

        
        private static $orderId = 0;

        
        public static function get( $element = "" ) {

                self::setItems();

                return self::getItems( $element );
        }

        
        private static function getItems( $element = "" ) {

                if ( !Helper::isNonEmptyArray( self::$items ) ) {
                        return '';
                }

                $dataStore = WC_Data_Store::load( 'customer-download-log' );

                $data = [];

                foreach ( self::$items as $item ) {

                        if ( 'download_log' === $element ) {

                                $logs = $dataStore->get_download_logs_for_permission( $item[ 'permission_id' ] );

                                if ( !Helper::isNonEmptyArray( $logs ) ) {
                                        $data[] = '';
                                        continue;
                                }

                                $downloadLog = [];

                                foreach ( $logs as $log ) {

                                        $userId = $log->get_user_id();

                                        $user = $userId > 0 ? \get_user_by( 'id', $userId ) : false;

                                        $downloadLog[] = [
                                                'user_email'      => ($user !== false && isset( $user->user_email ) ? $user->user_email : ''),
                                                'timestamp'       => $log->get_timestamp(),
                                                'user_ip_address' => $log->get_user_ip_address(),
                                        ];
                                }

                                $data[] = $downloadLog;
                                continue;
                        }

                        $data[] = isset( $item[ $element ] ) ? $item[ $element ] : '';
                }

                return $data;
        }

        
        private static function setItems() {

                if ( self::$orderId === DataCenter::getId() ) {
                        return;
                }

                self::$orderId = DataCenter::getId();

                $downloads = [];

                $dataStore         = WC_Data_Store::load( 'customer-download' );
                $customerDownloads = $dataStore->get_downloads(
                        [
                                'order_id' => DataCenter::getItem()->get_id(),
                                'return'   => 'permission_id,download_id,product_id,downloads_remaining,access_granted,access_expires,download_count',
                        ]
                );

                unset( $dataStore );

                if ( !Helper::isNonEmptyArray( $customerDownloads ) ) {
                        return;
                }

                foreach ( $customerDownloads as $customerDownload ) {

                        $downloadId = isset( $customerDownload[ 'download_id' ] ) ? $customerDownload[ 'download_id' ] : '';

                        $productId = isset( $customerDownload[ 'product_id' ] ) ? \absint( $customerDownload[ 'product_id' ] ) : '';

                        $itemName = '';

                        foreach ( DataCenter::getItem()->get_items() as $item ) {

                                if ( $productId === $item->get_product_id() || $productId === $item->get_variation_id() ) {
                                        $itemName = $item->get_name();
                                        break;
                                }
                        }

                        $product = \wc_get_product( $productId );

                        if ( !$product ) {
                                continue;
                        }

                        $file = $product->get_file( $downloadId );

                        if ( false === $file ) {
                                continue;
                        }

                        $customerDownload[ 'product' ]  = $product->get_name();
                        $customerDownload[ 'filename' ] = $file->get_name();
                        $customerDownload[ 'item' ]     = $itemName;
                        $downloads[]                    = $customerDownload;
                }


                self::$items = $downloads;
        }
}
