<?php



namespace WpieApp\Extensions\Export\WooCommerce\Controller\Items\Product;

use WpieApp\Core\Utils\DB;
use WpieApp\Modules\Export\Helpers\Helper;
use WpieApp\Modules\Export\Controller\Settings;
use WpieApp\Modules\Export\Controller\Items\Post\Engine as PostEngine;

defined( 'WPIE__FILE__' ) || exit;


class Engine extends PostEngine {

        
        const VARIATION_TYPE = "product_variation";

        
        protected static function processData( $data = [] ) {
                return DataCenter::init( $data, self::$action );
        }

        
        protected static function buildQuery() {

                $query = parent::buildQuery();

                if ( empty( Settings::getOption( 'wc/product/data' ) ) || in_array( 'variation', Settings::getOption( 'wc/product/data' ) ) ) {

                        $query[ 'post_type' ]   = ( isset( $query[ 'post_type' ] ) && is_array( $query[ 'post_type' ] ) ? $query[ 'post_type' ] : [ $query[ 'post_type' ] ]);
                        $query[ 'post_type' ][] = self::VARIATION_TYPE;
                }

                
                return \apply_filters( 'Wpie/Export/Product/Query', $query, Settings::getContentType() );
        }

        
        protected static function initFilter() {

                
                \do_action( 'Wpie/Export/Product/Filter/Start', Settings::getContentType() );

                Filter::init();

                
                \do_action( 'Wpie/Export/Product/Filter/Init', Settings::getContentType() );
                
                static::initBaseFilter();

                self::$where = Filter::getWhere();
                self::$join  = Filter::getJoin();
        }

        
        public static function filterWhere( $where = "" ) {

                $where = parent::filterWhere( $where );

                $where = self::addProductQuery( $where );

                return \apply_filters( 'Wpie/Export/Product/Filter/Where', $where );
        }

        
        public static function addProductQuery( $where = '' ) {

                $options = Settings::getOption( 'wc/product/data' );

                $requiredUpdate = (!Helper::isNonEmptyArray( $options )) || empty( array_diff( [ 'product', 'variation' ], $options ) );

                if ( !Helper::isNonEmptyArray( self::$where  ) && $requiredUpdate ) {
                        return $where;
                }

                $where .= self::excludeParent() . self::includeChild( $where );

                return $where;
        }

        
        public static function excludeParent() {

                if ( empty( Settings::getOption( 'wc/product/data' ) ) || in_array( 'product', Settings::getOption( 'wc/product/data' ) ) ) {
                        return "";
                }

                $tableAlias = Helper::randStr( "wpie_" );

                $termAlias    = $tableAlias . 'term';
                $termRelAlias = $tableAlias . 'tr';
                $termTaxAlias = $tableAlias . 'tt';

                $query = " SELECT object_id FROM " . DB::wpdb()->term_relationships . " AS " . $termRelAlias . "
                        LEFT JOIN " . DB::wpdb()->term_taxonomy . " AS " . $termTaxAlias . " ON (" . $termTaxAlias . ".term_taxonomy_id= " . $termRelAlias . ".term_taxonomy_id)
                        LEFT JOIN " . DB::wpdb()->terms . " AS " . $termAlias . " ON (" . $termAlias . ".term_id= " . $termTaxAlias . ".term_id)
                        WHERE " . $termTaxAlias . ".taxonomy = 'product_type' AND " . $termAlias . ".name = 'variable' ";

                return " AND (" . DB::wpdb()->posts . ".ID NOT IN (" . $query . "))";
        }

        
        public static function includeChild( $where = "" ) {


                if ( !empty( Settings::getOption( 'wc/product/data' ) ) && !in_array( 'variation', Settings::getOption( 'wc/product/data' ) ) ) {
                        return "";
                }

                $join = Helper::isNonEmptyArray( self::$join ) ? implode( ' ', self::$join ) : "";

                if ( Helper::isEmptyStr( $where ) ) {
                        $where = " AND " . DB::wpdb()->posts . ".`post_type` = 'product' AND " . DB::wpdb()->posts . ".`post_status` NOT IN ('trash','auto-draft') ";
                }

                $childQuery = "SELECT DISTINCT " . DB::wpdb()->posts . ".`ID` 
                               FROM " . DB::wpdb()->posts . " " . $join . " 
                               WHERE 1=1 " . $where;

                $query = DB::wpdb()->posts . ".`post_type` = 'product_variation' AND " . DB::wpdb()->posts . ".`post_parent` IN (" . $childQuery . ")";

                return " OR (" . $query . " ) ";
        }

}
