<?php



namespace WpieApp\Extensions\Export\YoastSeo\Controller\Items\Taxonomy\Fields;

use WpieApp\Modules\Export\Controller\Settings;
use WpieApp\Modules\Export\Controller\Items\Base\Fields\InterfaceField;

defined( 'WPIE__FILE__' ) || exit;


class Seo implements InterfaceField {

        
        const FIELD_TYPE = "yoast-seo";

        
        public static function get() {

                $fields = [
                        "title"      => \esc_html__( "Yoast SEO", 'wp-import-export' ),
                        "priority"   => 28,
                        "isFiltered" => false,
                        "data"       => self::getFields()
                ];

                
                return \apply_filters( 'Wpie/Export/Taxonomy/Fields/YoastSeo', $fields, Settings::getContentType() );
        }

        
        protected static function getFields() {

                return array_merge(
                        self::getGeneralFields(),
                        self::getSocialFields(),
                        self::getAdvancedFields()
                );
        }

        
        protected static function getGeneralFields() {

                return [
                        //General
                        [ 'label' => 'SEO Title', 'type' => self::FIELD_TYPE, 'element' => 'wpseo_title' ],
                        [ 'label' => 'SEO Description', 'type' => self::FIELD_TYPE, 'element' => 'wpseo_desc' ],
                        [ 'label' => 'SEO Focus keyphrase', 'type' => self::FIELD_TYPE, 'element' => 'wpseo_focuskw' ],
                        [ 'label' => 'SEO Related keyphrases', 'type' => self::FIELD_TYPE, 'element' => 'wpseo_focuskeywords' ],
                        [ 'label' => 'SEO Keyphrase Synonyms', 'type' => self::FIELD_TYPE, 'element' => 'wpseo_keywordsynonyms' ],
                        [ 'label' => 'SEO Is Cornerstone Content', 'type' => self::FIELD_TYPE, 'element' => 'wpseo_is_cornerstone' ],
                        [ 'label' => 'SEO Score', 'type' => self::FIELD_TYPE, 'element' => 'wpseo_content_score' ]
                ];
        }

        
        protected static function getSocialFields() {

                return [
                        //Facebook
                        [ 'label' => 'SEO Facebook Title', 'type' => self::FIELD_TYPE, 'element' => 'wpseo_opengraph-title' ],
                        [ 'label' => 'SEO Facebook Description', 'type' => self::FIELD_TYPE, 'element' => 'wpseo_opengraph-description' ],
                        [ 'label' => 'SEO Facebook Image', 'type' => self::FIELD_TYPE, 'element' => 'wpseo_opengraph-image' ],
                        //Twitter
                        [ 'label' => 'SEO Twitter Title', 'type' => self::FIELD_TYPE, 'element' => 'wpseo_twitter-title' ],
                        [ 'label' => 'SEO Twitter Description', 'type' => self::FIELD_TYPE, 'element' => 'wpseo_twitter-description' ],
                        [ 'label' => 'SEO Twitter Image', 'type' => self::FIELD_TYPE, 'element' => 'wpseo_twitter-image' ],
                ];
        }

        
        protected static function getAdvancedFields() {

                return [
                        // Robots Meta
                        [ 'label' => 'SEO Robots No Index', 'type' => self::FIELD_TYPE, 'element' => 'wpseo_noindex' ],
                        [ 'label' => 'SEO Breadcrumbs Title', 'type' => self::FIELD_TYPE, 'element' => 'wpseo_bctitle' ],
                        [ 'label' => 'SEO Canonical URL', 'type' => self::FIELD_TYPE, 'element' => 'wpseo_canonical' ]
                ];
        }
}
