<?php



namespace WpieApp\Extensions\Import\Acf\Controller\Items\Base\Engines;

use WpieApp\Modules\Import\Helpers\Helper;
use WpieApp\Modules\Import\Controller\Items\Base\DataCenter;

defined( 'WPIE__FILE__' ) || exit;


class ACF {

        
        protected static $prefix = '';

        
        public static function get( $prefix = '' ) {

                static::$prefix = $prefix;

                $fields = DataCenter::getFieldSetting( 'acf' );

                if ( !Helper::isNonEmptyArray( $fields ) ) {
                        return;
                }

                $acf = [];

                foreach ( $fields as $key => $value ) {

                        if ( isset( $value[ 'data' ] ) ) {

                                if ( DataCenter::canUpdate( 'acf/' . $key ) ) {
                                        $acf[ $key ] = self::processField( $key, $value );
                                }

                                continue;
                        }

                        $acfData = self::processCloneField( $key, $value );

                        if ( Helper::isNonEmptyArray( $acfData ) ) {
                                $acf[ $key ] = $acfData;
                        }
                }

                return $acf;
        }

        
        protected static function processCloneField( $fieldKey = '', $fields = [] ) {

                if ( !Helper::isNonEmptyArray( $fields ) ) {
                        return [];
                }

                $cloneField = self::getField( $fieldKey );

                if ( !is_array( $cloneField ) || !isset( $cloneField[ 'type' ] ) || 'clone' !== $cloneField[ 'type' ] ) {
                        return [];
                }
                if ( !isset( $cloneField[ 'sub_fields' ] ) || !is_array( $cloneField[ 'sub_fields' ] ) ) {
                        return [];
                }

                $acf = [];

                foreach ( $cloneField[ 'sub_fields' ] as $field ) {

                        $id = isset( $field[ 'ID' ] ) ? $field[ 'ID' ] : '';

                        $key = isset( $field[ 'key' ] ) ? $field[ 'key' ] : '';

                        $val = isset( $fields[ $key ] ) ? $fields[ $key ] : '';

                        if ( DataCenter::canUpdate( 'acf/' . $fieldKey . '/' . $key ) ) {
                                $acf[ $key ] = self::processField( $key, $val, $id );
                        }
                }

                return $acf;
        }

        
        protected static function processField( $key = '', $value = [], $id = 0 ) {

                $fieldKey = (is_numeric( $id ) && 0 < $id ) ? $id : $key;

                $data = isset( $value[ 'data' ] ) ? DataCenter::getFileData( $value[ 'data' ] ) : '';

                if ( Helper::isEmpty( $data ) ) {
                        return '';
                }

                $field = self::getField( $fieldKey );

                if ( false === $field ) {
                        return '';
                }

                $type = isset( $field[ 'type' ] ) ? $field[ 'type' ] : '';

                $formatValue = static::formatValue( $field, $data );

                if ( \is_wp_error( $formatValue ) ) {
                        return '';
                }

                if ( 'icon_picker' === $type && !Helper::isNonEmptyArray( $formatValue ) ) {
                        return '';
                }

                return $formatValue;
        }

        
        protected static function formatValue( $field = [], $data = '' ) {

                if ( Helper::isEmpty( $data ) ) {
                        return;
                }

                $type = isset( $field[ 'type' ] ) ? $field[ 'type' ] : '';

                if ( Basic::isAllowed( $type ) ) {
                        return Basic::get( $field, $data, static::$prefix );
                }
                if ( Content::isAllowed( $type ) ) {
                        return Content::get( $field, $data, static::$prefix );
                }
                if ( Choice::isAllowed( $type ) ) {
                        return Choice::get( $field, $data, static::$prefix );
                }
                if ( Relational::isAllowed( $type ) ) {
                        return Relational::get( $field, $data, static::$prefix );
                }
                if ( Advanced::isAllowed( $type ) ) {
                        return Advanced::get( $field, $data, static::$prefix );
                }
                if ( Layout::isAllowed( $type ) ) {
                        return Layout::get( $field, $data, static::$prefix );
                }
                if ( Jquery::isAllowed( $type ) ) {
                        return Jquery::get( $field, $data, static::$prefix );
                }
                if ( AcfFields::isAllowed( $type ) ) {
                        return AcfFields::get( $field, $data, static::$prefix );
                }
                if ( WP::isAllowed( $type ) ) {
                        return WP::get( $field, $data, static::$prefix );
                }

                return $data;
        }

        
        protected static function getField( $key = '' ) {

                if ( function_exists( "\acf_get_field" ) ) {
                        return \acf_get_field( $key );
                } else {
                        return \apply_filters( 'acf/load_field', false, $key );
                }
        }
}
