<?php



namespace WpieApp\Extensions\Import\AllInOneSeo\Controller\Items\User\Engines;

use WpieApp\Modules\Import\Helpers\Helper;
use WpieApp\Modules\Import\Controller\Items\User\DataCenter;
use WpieApp\Modules\Import\Controller\Items\Base\Logger;

defined( 'WPIE__FILE__' ) || exit;


class Social {

        
        public static function get() {

                if ( \version_compare( AIOSEO_VERSION, '4.2.2', '<' ) ) {
                        return Legacy::get();
                }

                $data = [];

                if ( DataCenter::canUpdate( 'allinoneseo/social/profiles_additional_urls' ) ) {
                        $data[ "aioseo_profiles_additional_urls" ] = DataCenter::get( "allinoneseo/social/profiles_additional_urls/data" );
                }

                if ( DataCenter::canUpdate( 'allinoneseo/social/profiles_same_username' ) ) {
                        $data[ "aioseo_profiles_same_username" ] = self::getSameUsername();
                }

                return array_merge( $data, self::getUrls() );
        }

        
        private static function getSameUsername() {

                $data = DataCenter::get( "allinoneseo/social/profiles_same_username/data" );

                if ( !Helper::isNonEmptyStr( $data ) ) {

                        return [];
                }

                $sameUsername = Helper::jsonDecode( $data, true );

                if ( \is_wp_error( $sameUsername ) ) {

                        Logger::log( "<strong>" . __( "ERROR", 'wp-import-export' ) . "</strong> : " . $sameUsername->get_error_message() );

                        return [];
                }

                return $sameUsername;
        }

        
        protected static function getUrls() {

                $fields = [ "facebook_page_url", "twitter_url", "instagram_url", "tiktok_url", "pinterest_url", "youtube_url", "linkedin_url", "tumblr_url", "yelp_page_url", "sound_cloud_url", "wikipedia_url", "myspace_url", "word_press_url" ];

                $data = [];

                foreach ( $fields as $field ) {

                        if ( !DataCenter::canUpdate( 'allinoneseo/social/' . $field ) ) {
                                continue;
                        }

                        $data[ "aioseo_" . $field ] = \esc_url_raw( DataCenter::get( 'allinoneseo/social/' . $field . '/data' ) );
                }

                return $data;
        }
}
