<?php



namespace WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order;

use WpieApp\Modules\Import\Helpers\Helper;
use WpieApp\Modules\Import\Controller\Settings;
use WpieApp\Modules\Import\Controller\Items\Base\Logger;
use WpieApp\Modules\Import\Controller\Items\Base\Engine as BaseEngine;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines\Notification;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines\Standard;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines\Other;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines\Customer;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines\Payment;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines\Shipping;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines\Billing;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines\Product;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines\Tax;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines\Fee;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines\Download;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines\Coupon;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines\ShippingMethod;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines\Refund;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines\Notes;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines\Meta;

defined( 'WPIE__FILE__' ) || exit;


class Engine extends BaseEngine {

        
        protected static function preStart() {
                Notification::set();
        }

        
        public static function import() {

                \do_action( "Wpie/Import/Order/Engine/Start" );

                if ( !static::insertItem() ) {
                        return false;
                }

                static::afterAdd();

                static::updateItem();

                \do_action( "Wpie/Import/Order/Engine/End" );

                return true;
        }

        
        protected static function afterAdd() {

                \do_action( "Wpie/Import/Order/Engine/Add/After" );

                parent::afterAdd();
        }

        
        protected static function setItem( $id = 0 ) {

                DataCenter::setId( $id );

                DataCenter::setItem( \wc_get_order( $id ) );
        }

        
        protected static function updateItem() {

                DataCenter::getItem()->set_object_read( true );

                Notes::log();

                static::updateMeta();

                Standard::update();

                Other::update();

                Customer::update();

                Payment::update();

                Billing::update();

                Shipping::update();

                Tax::update();

                Product::update();

                Fee::update();

                ShippingMethod::update();

                Coupon::update();

                DataCenter::getItem()->calculate_totals();

                Download::update();

                Refund::update();

                Notes::update();
        }

        
        protected static function updateMeta() {

                Meta::import();
        }

        
        protected static function insertItem() {

                $id = ( DataCenter::isNewItem() ? self::insert() : self::update());

                if ( false === $id ) {
                        return false;
                }

                if ( \is_wp_error( $id ) ) {

                        Logger::log( "<strong>" . __( "ERROR", 'wp-import-export' ) . "</strong> : " . $id->get_error_message() );

                        return false;
                }

                static::setItem( $id );

                return true;
        }

        
        protected static function insert() {

                $status = Standard::getStatus();

                if ( !Helper::isNonEmptyStr( $status ) ) {

                        Logger::log( "<strong>" . __( "ERROR", 'wp-import-export' ) . "</strong> : " . __( "Invalid Order status", 'wp-import-export' ) );

                        return false;
                }

                $order = \wc_create_order( [ 'status' => $status ] );

                if ( \is_wp_error( $order ) ) {

                        Logger::log( "<strong>" . __( "ERROR", 'wp-import-export' ) . "</strong> : " . $order->get_error_message() );

                        return false;
                }

                return $order->get_id();
        }

        
        protected static function update() {

                return DataCenter::getId();
        }

        
        public static function searchDuplicates() {

                $searchMethod = DataCenter::get( "search/method" );

                $id = 0;

                switch ( $searchMethod ) {

                        case "key":
                                $key = DataCenter::get( "search/key" );

                                if ( Helper::isNonEmptyStr( $key ) ) {
                                        $id = AutoSearch::search( $key );
                                } else {
                                        $id = AutoSearch::get();
                                }
                                break;

                        default:
                                $id = AutoSearch::get();
                                break;
                }

                return $id;
        }
}
