<?php



namespace WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\Engines;

use WpieApp\Core\Utils\Date;
use WpieApp\Modules\Import\Helpers\Helper;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Order\DataCenter;

defined( 'WPIE__FILE__' ) || exit;


class Standard {

        
        public static function update() {

                self::updateStatus();
                self::updateKey();
                self::updateDate();
                self::updateDateCompleted();
                self::updateTotal();
                self::updateOrderNumber();
        }

        
        private static function updateTotal() {

                if ( !DataCenter::canUpdate( "standard/total" ) ) {
                        return;
                }

                $total = DataCenter::get( "standard/total/data" );

                if ( '' === $total || !is_numeric( $total ) ) {
                        return;
                }

                DataCenter::getItem()->set_total( $total );
        }

        
        private static function updateDate() {

                if ( !DataCenter::canUpdate( "standard/date" ) ) {
                        return;
                }

                $date = DataCenter::get( "standard/date/data" );

                $date = ( "now" === $date || !Helper::isNonEmptyStr( $date ) || \strtotime( $date ) === false ) ? Date::now() : $date;

                $date = \strtotime( Date::get( $date ) );

                DataCenter::getItem()->set_date_created( $date );

                unset( $date );
        }

        
        private static function updateDateCompleted() {

                if ( !DataCenter::canUpdate( "standard/date-completed" ) ) {
                        return;
                }

                $date = DataCenter::get( "standard/date-completed/data" );

                $date = ( "now" === $date || !Helper::isNonEmptyStr( $date ) || \strtotime( $date ) === false ) ? Date::now() : $date;

                $date = \strtotime( Date::get( $date ) );

                DataCenter::getItem()->set_date_completed( $date );

                unset( $date );
        }

        
        private static function updateStatus() {

                $status = self::getStatus();

                if ( !Helper::isNonEmptyStr( $status ) ) {
                        return;
                }

                DataCenter::getItem()->set_status( $status );
        }

        
        private static function updateKey() {

                if ( !DataCenter::canUpdate( "standard/key" ) ) {
                        return;
                }

                $key = DataCenter::get( "standard/key/data" );

                if ( !Helper::isNonEmptyStr( $key ) ) {
                        return;
                }

                DataCenter::getItem()->set_order_key( $key );
        }

        
        private static function updateOrderNumber() {

                if ( !DataCenter::canUpdate( "standard/order-number" ) ) {
                        return;
                }

                $orderNumber = DataCenter::get( "standard/order-number/data" );

                if ( !Helper::isNonEmptyStr( $orderNumber ) ) {
                        return;
                }

                DataCenter::getItem()->update_meta_data( '_wpie_order_number', $orderNumber );
        }

        
        public static function getStatus() {

                if ( !DataCenter::canUpdate( 'standard/status' ) ) {
                        return false;
                }

                $statuses = \wc_get_order_statuses();

                $input = strtolower( DataCenter::get( "standard/status/data" ) );

                $input = ('wc-' === substr( $input, 0, 3 ) ? $input : 'wc-' . $input);

                return Helper::autoFixStr( $input, array_keys( $statuses ), 7 );
        }
}
