<?php



namespace WpieApp\Extensions\Import\WooCommerce\Controller\Items\Product\Engines\Attributes;

use WC_Product_Attribute;
use WpieApp\Modules\Import\Helpers\Helper;
use WpieApp\Modules\Import\Controller\Items\Post\DataCenter;
use WpieApp\Modules\Import\Controller\Items\Base\Logger;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Product\Engines\Product;
use WpieApp\Extensions\Import\WooCommerce\Controller\Items\Attribute\Engines\Attribute;

defined( 'WPIE__FILE__' ) || exit;


class Base {

        
        private static $cache = [];

        
        private static $attributes = null;

        
        public static function getRawAttributes() {

                if ( isset( self::$cache[ DataCenter::getKey() ] ) ) {
                        return self::$cache[ DataCenter::getKey() ];
                }

                $path = 'attribute/';

                $nameList = DataCenter::get( $path . 'name' );

                if ( !Helper::isNonEmptyArray( $nameList ) ) {
                        return [];
                }

                $attributes = [];

                for ( $index = 1; $index <= count( $nameList ); $index++ ) {

                        $name = DataCenter::get( $path . 'name/' . $index );

                        $val = DataCenter::get( $path . 'value/' . $index );

                        if ( !Helper::isNonEmptyStr( $name ) || !Helper::isNonEmptyStr( $val ) ) {
                                continue;
                        }

                        $separator = DataCenter::get( $path . 'separator/' . $index . '/data' );
                        $separator = (!Helper::isNonEmptyStr( $separator ) ) ? '||' : $separator;

                        $val = Helper::strToArr( trim( $val ), $separator );

                        $slug = DataCenter::get( $path . 'slug/' . $index . '/data' );
                        $slug = (!Helper::isNonEmptyStr( $slug ) || $slug === 'auto') ? $name : $slug;

                        $isTaxonomy = DataCenter::get( $path . 'is_taxonomy/' . $index . '/data' );
                        $isTaxonomy = !Helper::isNonEmptyStr( $isTaxonomy ) || Helper::detectBoolVal( $isTaxonomy ) !== false;

                        $taxonomy = (($isTaxonomy ) ? \wc_attribute_taxonomy_name( $slug ) : \sanitize_title( $slug ) );
                        $position = DataCenter::get( $path . 'position/' . $index . '/data' );

                        $isVisible   = DataCenter::get( $path . 'is_visible/' . $index . '/data' );
                        $isVariation = DataCenter::get( $path . 'in_variations/' . $index . '/data' );

                        $attributes[ $taxonomy ] = [
                                'name'         => $name,
                                'slug'         => $slug,
                                'value'        => $val,
                                'taxonomy'     => $taxonomy,
                                'is_taxonomy'  => $isTaxonomy,
                                'is_visible'   => !Helper::isNonEmptyStr( $isVisible ) || Helper::detectBoolVal( $isVisible ) !== false,
                                'is_variation' => !Helper::isNonEmptyStr( $isVariation ) || Helper::detectBoolVal( $isVariation ) !== false,
                                'position'     => ($position == 'auto' || !is_numeric( $position )) ? 0 : $position
                        ];
                }

                self::$cache[ DataCenter::getKey() ] = $attributes;

                if ( 5 < count( self::$cache ) ) {
                        array_shift( self::$cache );
                }

                return $attributes;
        }

        
        public static function getExistingAttributes() {

                if ( DataCenter::isNewItem() || DataCenter::getFieldSetting( 'attributes/options/manage-existing' ) !== "append" ) {
                        return [];
                }

                return Product::getProduct()->get_attributes( 'edit' );
        }

        
        public static function generate( $data = [] ) {

                if ( !Helper::isNonEmptyArray( $data ) ) {
                        return false;
                }

                $attributeData = array_merge(
                        [
                                'name'         => '',
                                'slug'         => '',
                                'value'        => '',
                                'taxonomy'     => '',
                                'position'     => 0,
                                'is_visible'   => true,
                                'is_variation' => true,
                                'is_taxonomy'  => true,
                        ],
                        ( array ) $data
                );

                $attributeId = 0;

                if ( $attributeData[ 'is_taxonomy' ] ) {

                        $attributeId = Attribute::get( [ 'name' => $attributeData[ 'name' ], 'slug' => $attributeData[ 'slug' ] ] );

                        if ( \is_numeric( $attributeId ) && \absint( $attributeId ) > 0 ) {
                                $attributeData[ 'name' ]  = $attributeData[ 'taxonomy' ];
                                $attributeData[ 'value' ] = Term::get( $attributeData[ 'value' ], $attributeData[ 'taxonomy' ] );
                        }
                }

                $attribute = new WC_Product_Attribute();
                $attribute->set_id( $attributeId );
                $attribute->set_name( $attributeData[ 'name' ] );
                $attribute->set_options( $attributeData[ 'value' ] );
                $attribute->set_position( $attributeData[ 'position' ] );
                $attribute->set_visible( $attributeData[ 'is_visible' ] );
                $attribute->set_variation( $attributeData[ 'is_variation' ] );

                return $attribute;
        }
}
