<?php



namespace WpieApp\Extensions\Import\WooCommerce\Controller\Items\Product\Engines;

use WpieApp\Modules\Import\Helpers\Helper;
use WpieApp\Modules\Import\Controller\Items\Post\Search;
use WpieApp\Modules\Import\Controller\Items\Post\DataCenter;

defined( 'WPIE__FILE__' ) || exit;


class LinkedProducts {

        
        public static function get() {

                $data = [];

                if ( DataCenter::canUpdate( "linked-products/upsells" ) ) {
                        $data[ 'upsell_ids' ] = self::findProduct( 'upsells' );
                }

                if ( DataCenter::canUpdate( "linked-products/cross-sells" ) ) {
                        $data[ 'cross_sell_ids' ] = self::findProduct( 'cross-sells' );
                }

                if ( DataCenter::canUpdate( "linked-products/grouped" ) ) {
                        $data[ 'children' ] = self::findProduct( 'grouped' );
                }

                return $data;
        }

        
        private static function findProduct( $key = '' ) {

                $data = DataCenter::get( 'linked-products/' . $key . '/data' );

                if ( !Helper::isNonEmptyStr( $data ) ) {
                        return [];
                }

                $separator = DataCenter::get( 'linked-products/' . $key . '/separator' );

                $inputs = Helper::strToArr( $data, Helper::isNonEmptyStr( $separator ) ? $separator : '|~|' );

                $products = [];

                foreach ( $inputs as $input ) {

                        $id = self::search( $input );

                        if ( $id < 1 ) {
                                continue;
                        }
                        $products[] = $id;
                }

                return $products;
        }

        
        private static function search( $data = '' ) {

                if ( !Helper::isNonEmptyStr( $data ) ) {
                        return 0;
                }

                $postTypes = [ 'product', 'product_variation' ];

                $id = Search::getPostIdByMeta( '_sku', $data, $postTypes );

                if ( $id > 0 ) {
                        return $id;
                }

                $options = [
                        [
                                "args" => [
                                        [ "key" => 'title', "value" => $data ],
                                        [ "key" => 'content', "value" => $data, "relation" => "or" ],
                                        [ "key" => 'slug', "value" => $data, "relation" => "or" ],
                                        [ "key" => 'id', "value" => $data, "relation" => "or", 'is_int' => true, 'conditions' => [ 'positive_number_only' ] ],
                                ]
                        ],
                        [
                                "relation" => "and",
                                "args"     => [
                                        [ "key" => "type", "value" => $postTypes ]
                                ]
                        ]
                ];

                return Search::getId( $options );
        }
}
