<?php



namespace WpieApp\Extensions\Import\WooCommerce\Controller\Items\Product\Engines;

use WC_Product_Factory;
use WpieApp\Modules\Import\Helpers\Helper;
use WpieApp\Modules\Import\Controller\Items\Base\Logger;
use WpieApp\Modules\Import\Controller\Items\Post\DataCenter;

defined( 'WPIE__FILE__' ) || exit;


class Product {

        
        private static $product = null;

        
        public static function import( $product = null ) {

                self::$product = null;

                if ( is_a( $product, '\WC_Product' ) ) {
                        self::$product = $product;
                } elseif ( !self::setProduct() ) {
                        return false;
                }

                if ( !self::isValidProduct() ) {
                        return false;
                }

                self::processData();

                self::save();

                return true;
        }

        
        private static function setProduct() {

                $product = null;

                if ( DataCenter::isNewItem() ) {

                        $product = self::getObject( self::getType(), DataCenter::getId() );
                } else {
                        $product = \wc_get_product( DataCenter::getId() );
                }

                if ( !is_a( $product, '\WC_Product' ) ) {
                        Logger::log( "<strong>" . __( "ERROR", 'wp-import-export' ) . "</strong> : " . __( "Product cannot be loaded", 'wp-import-export' ) );
                        return false;
                }

                self::$product = $product;

                return true;
        }

        
        public static function getProduct() {

                return self::$product;
        }

        
        public static function resetProduct( $needUpdate = false ) {

                if ( $needUpdate ) {
                        self::save();
                }

                $type = self::$product->get_type();

                self::$product = self::getObject( $type, DataCenter::getId() );
        }

        
        private static function getType() {

                $productTypes = \wc_get_product_types();

                $type = '';

                if ( DataCenter::canUpdate( "type/product-type" ) ) {
                        $type = DataCenter::get( "type/product-type/data" );
                }

                $type = Helper::autoFixStr( $type, array_keys( $productTypes ), false );

                if ( !Helper::isNonEmptyStr( $type ) || !is_array( $productTypes ) || !isset( $productTypes[ $type ] ) ) {
                        $type = 'simple';
                }

                return $type;
        }

        
        private static function isValidProduct() {

                if ( !DataCenter::isNewItem() || !self::$product->is_type( 'variation' ) ) {
                        return true;
                }

                $attributes = Attributes::get();

                if ( !Helper::isNonEmptyArray( $attributes ) || !isset( $attributes[ 'attributes' ] ) || !Helper::isNonEmptyArray( $attributes[ 'attributes' ] ) ) {
                        Logger::log( "<strong>" . __( "ERROR", 'wp-import-export' ) . "</strong> : " . __( "can't create variation without attributes.", 'wp-import-export' ) );
                        Logger::log( "<strong>" . __( "DELETED", 'wp-import-export' ) . "</strong> : " . __( "variation without attributes.", 'wp-import-export' ) );
                        self::$product->delete( true );
                        return false;
                }

                return true;
        }

        
        private static function processData() {

                self::$product->set_props(
                        array_merge(
                                Price::get(),
                                Stock::get(),
                                Gallery::get(),
                                DataAndVisibility::get(),
                                General::get(),
                                Downloadable::get(),
                                Shipping::get(),
                                LinkedProducts::get(),
                                Advanced::get(),
                                Variation::get(),
                                Attributes::get()
                        )
                );
        }

        
        private static function save() {

                \do_action( "Wpie/Import/Product/Insert/Before", DataCenter::getId() );

                self::$product->save();

                \do_action( "Wpie/Import/Product/Insert/After", DataCenter::getId() );
        }

        
        public static function getObject( $type, $id ) {

                if ( function_exists( '\wc_get_product_object' ) ) {
                        return \wc_get_product_object( $type, $id );
                }

                $classname = WC_Product_Factory::get_product_classname( $id, $type );

                return new $classname( $id );
        }
}
