<?php



namespace WpieApp\Modules\Export\Controller\Items\Post\Fields;

use WpieApp\Core\Helpers\Helper;
use WpieApp\Modules\Export\Controller\Settings;
use WpieApp\Modules\Export\Controller\Items\Base\Fields\InterfaceField;

defined( 'WPIE__FILE__' ) || exit;


class Taxonomy implements InterfaceField {

        
        const FIELD_TYPE = "taxonomy";

        
        public static function get() {

                $fields = [
                        "title"    => \esc_html__( "Taxonomy", 'wp-import-export' ),
                        "priority" => 20,
                        "data"     => static::getFields( Settings::getContentType() )
                ];

                
                return \apply_filters( 'Wpie/Export/Post/Fields/Taxonomy', $fields, Settings::getContentType() );
        }

        
        public static function getFields( $postType = "post", $fieldType = self::FIELD_TYPE ) {

                global $wp_taxonomies;

                if ( !Helper::isNonEmptyArray( $wp_taxonomies ) || !Helper::isNonEmptyStr( $postType ) ) {
                        return [];
                }

                $preList = [
                        'product_type'       => __( "Product Type", 'wp-import-export' ),
                        'product_visibility' => __( "Product Visibility", 'wp-import-export' )
                ];

                $taxonomies = [];

                $excludes = static::getExcludesTaxonomy( $postType );

                foreach ( $wp_taxonomies as $key => $tax ) {

                        if (
                                $key === "post_format" ||
                                !isset( $tax->name ) ||
                                strpos( $tax->name, "pa_" ) === 0 ||
                                !in_array( $postType, $tax->object_type ) ||
                                ( Helper::isNonEmptyArray( $excludes ) && in_array( $tax->name, $excludes ))
                        ) {
                                continue;
                        }

                        if ( isset( $preList[ $tax->name ] ) ) {
                                $label = $preList[ $tax->name ];
                        } else {
                                $label = isset( $tax->label ) ? $tax->label : $tax->name;
                        }

                        $taxonomies[] = [
                                'label'             => $label,
                                'type'              => $fieldType,
                                'element'           => $tax->name,
                                'excludeConditions' => [ 'is_empty', 'is_not_empty' ],
                                'hierarchical'      => $tax->hierarchical
                        ];
                }

                return $taxonomies;
        }

        
        protected static function getExcludesTaxonomy( $type = "" ) {

                
                $excludes = apply_filters( 'Wpie/Export/Post/Fields/Taxonomy/Excludes', [], $type );

                return Helper::isNonEmptyArray( $excludes ) ? array_unique( $excludes ) : [];
        }
}
