<?php



namespace WpieApp\Modules\Export\Controller\Items\User\Fields;

use WpieApp\Core\Utils\DB;
use WpieApp\Modules\Export\Helpers\Helper;
use WpieApp\Modules\Export\Controller\Settings;
use WpieApp\Modules\Export\Controller\Items\Base\Fields\InterfaceField;
use WpieApp\Modules\Export\Controller\Items\Base\Fields\Meta as BaseMeta;

defined( 'WPIE__FILE__' ) || exit;


class Meta extends BaseMeta implements InterfaceField {

        
        public static function get() {

                
                return \apply_filters( 'Wpie/Export/User/Fields/Meta', parent::get(), Settings::getContentType() );
        }

        
        protected static function getMetaKeys( $contentType = '' ) {
                
                $limit = \apply_filters( 'Wpie/Export/User/Fields/Meta/Limit', 500 );

                
                $types = \apply_filters( 'Wpie/Export/User/Fields/Meta/Type', Settings::getContentType() );

                $excludes = static::getExcludesMetaQuery( $types );

                $keys = DB::wpdb()->get_col(
                        DB::wpdb()->prepare(
                                "SELECT DISTINCT `usermeta`.`meta_key`
				FROM " . DB::wpdb()->usermeta . " AS `usermeta`
				LEFT JOIN " . DB::wpdb()->users . " AS `users` ON ( `users`.`ID` = `usermeta`.`user_id` )
				WHERE `usermeta`.`meta_value` != '' " .
                                $excludes
                                . " ORDER BY `usermeta`.`meta_key`
				LIMIT %d",
                                $limit
                        )
                );

                return $keys;
        }

        
        protected static function getExcludesMeta( $contentType = '' ) {

                $caps = DB::wpdb()->get_blog_prefix() . 'capabilities';

                $excludes = array_merge(
                        [ 'first_name', 'last_name', 'nickname', 'description', 'locale', $caps ],
                        parent::getExcludesMeta()
                );

                
                $excludes = apply_filters( 'Wpie/Export/User/Fields/Meta/Excludes', $excludes, $contentType );

                return Helper::isNonEmptyArray( $excludes ) ? array_unique( $excludes ) : [];
        }

        
        protected static function getExcludesMetaQuery( $contentType = "" ) {

                $query = parent::getExcludesMetaQuery();

                $keys = DB::prepareData( static::getExcludesMeta( $contentType ) );

                if ( Helper::isNonEmptyStr( $keys ) ) {
                        $query[] = " AND `usermeta`.`meta_key` NOT IN ( " . $keys . " ) ";
                }

                
                $query = \apply_filters( 'Wpie/Export/User/Fields/Meta/Query/Where', $query, $contentType );

                return Helper::isNonEmptyArray( $query ) ? implode( " ", array_unique( $query ) ) : "";
        }
}
