<?php



namespace WpieApp\Modules\Export\Controller;

use WpieApp\Core\Utils\Date;
use WpieApp\Core\Office\Converter;
use WpieApp\Core\Utils\Archive;
use WpieApp\Modules\Export\Helpers\Helper;
use WpieApp\Modules\Export\Controller\Items\Manager as Items;

defined( 'WPIE__FILE__' ) || exit;


class Utils {

        
        public static function preExport() {

                self::setName();

                self::setFields();
        }

        
        public static function setFields() {

                if ( "" !== Settings::getOption( 'fields' ) ) {
                        return;
                }

                $fieldGroups = Items::get( 'fields' );

                if ( !Helper::isNonEmptyArray( $fieldGroups ) ) {
                        return;
                }

                $fields = [];

                foreach ( $fieldGroups as $fieldGroup ) {

                        if ( !Helper::isNonEmptyArray( $fieldGroup ) || !isset( $fieldGroup[ 'data' ] ) ) {
                                continue;
                        }

                        if ( isset( $fieldGroup[ 'isExported' ] ) && $fieldGroup[ 'isExported' ] === false ) {
                                continue;
                        }

                        foreach ( $fieldGroup[ 'data' ] as $field ) {

                                if ( !Helper::isNonEmptyArray( $field ) ) {
                                        continue;
                                }

                                if ( isset( $field[ 'isExported' ] ) && $field[ 'isExported' ] === false ) {
                                        continue;
                                }

                                $fields[] = $field;
                        }
                }

                Settings::setOptions( [ 'fields' => [ "data" => $fields, 'available' => array_keys( $fields ) ] ] );
        }

        
        public static function setName() {

                if ( Helper::isNonEmptyStr( Settings::get( 'file' ) ) ) {
                        return;
                }

                $filename = Settings::getOption( 'options/file/name' );

                $contentType = Helper::getContentTypeLabel( Settings::get( 'category' ) . '/' . Settings::get( 'content_type' ) );

                Settings::setOption( 'info/contentTypeLabel', $contentType );

                $default = Helper::sanitizeFilename( $contentType . ' Export ' . Date::get( '', 'Y M d His' ), 'csv' );

                $name = Helper::isNonEmptyStr( $filename ) ? Helper::sanitizeFilename( $filename, 'csv' ) : '';

                Settings::set( 'file', Helper::isNonEmptyStr( $name ) ? $name : $default );
        }

        
        public static function download( $isBundle = false ) {

                $format = $isBundle ? 'zip' : Helper::sanitizeKey( Settings::getOption( 'options/file/format' ) );

                $filename = pathinfo( Settings::get( 'file' ), PATHINFO_FILENAME ) . "." . (Helper::isNonEmptyStr( $format ) ? $format : "csv");

                Helper::download( Helper::pathJoin( Settings::getCurrentDir( 'file' ), $filename ) );
        }

        
        public static function preCompleteTasks() {

                self::setImportSettings();

                self::createBundle();

                $tasks = self::changeFormat();

                if ( \is_wp_error( $tasks ) ) {
                        return $tasks;
                }
                if ( $tasks ) {
                        self::saveExtraCopy();
                        Settings::set( 'status', "completed" );
                        return "completed";
                }

                return "pending";
        }

        
        public static function saveExtraCopy() {

                $extraCopy = Settings::getOption( 'options/file/extra-copy' );

                if ( $extraCopy === "" || absint( $extraCopy ) !== 1 ) {
                        return;
                }

                $format = Helper::sanitizeKey( Settings::getOption( 'options/file/format' ) );

                $filename = pathinfo( Settings::get( 'file' ), PATHINFO_FILENAME ) . "." . (Helper::isNonEmptyStr( $format ) ? $format : "csv");

                $source = Helper::pathJoin( Settings::getCurrentDir( "file" ), $filename );

                $destination = Helper::pathJoin( WPIE_UPLOAD_TEMP_DIR, $filename );

                Helper::duplicate( $source, $destination );
        }

        
        public static function setImportSettings() {

                if ( "completed" === Settings::getOption( 'tasks/import/settings' ) ) {
                        return;
                }

                $file = Helper::pathJoin( Settings::getCurrentDir( "file" ), 'config.json' );

                $data = Items::get( 'import-config' );

                Helper::getFilesystem()->put_contents( $file, Helper::isNonEmptyArray( $data ) ? Helper::jsonEncode( $data ) : "" );

                Settings::setOption( 'tasks/import/settings', "completed" );
        }

        
        public static function createBundle() {

                if ( "completed" === Settings::getOption( 'tasks/bundle' ) ) {
                        return;
                }

                $archive = Helper::pathJoin( Settings::getCurrentDir( "file" ), Helper::sanitizeFilename( Settings::get( 'file' ), "zip" ) );

                //compress files
                $files = [
                        Settings::get( 'file' ) => Settings::getFilePath(),
                        'config.json'           => Helper::pathJoin( Settings::getCurrentDir( "file" ), 'config.json' )
                ];

                Archive::create( $files, $archive );

                Settings::setOption( 'tasks/bundle', "completed" );
        }

        
        public static function changeFormat() {

                if ( "completed" === Settings::getOption( 'tasks/convert' ) ) {
                        return true;
                }

                $format = Helper::sanitizeKey( Settings::getOption( 'options/file/format' ) );

                if ( !Helper::isNonEmptyStr( $format ) || $format === "csv" ) {

                        Settings::setOption( 'tasks/convert', "completed" );

                        return true;
                }

                $destFile = Helper::sanitizeFilename( Settings::get( "file" ), $format );

                $dest = Helper::pathJoin( dirname( Settings::getFilePath() ), $destFile );

                $status = Converter::fromCsv( Settings::getFilePath(), $format, $dest );

                if ( is_wp_error( $status ) ) {
                        return $status;
                }

                Settings::setOption( 'tasks/convert', "completed" );

                return true;
        }

        
        public static function saveFields() {

                $fields = Settings::getOption( 'fields/data' );

                if ( Helper::isNonEmptyStr( $fields ) ) {
                        Settings::setOption( 'fields/data', Helper::jsonDecode( $fields, true ) );
                } else {
                        Settings::setOption( 'fields', "" );
                }

                $customFields = Settings::getOption( 'fields/custom' );
                if ( Helper::isNonEmptyStr( $customFields ) ) {
                        Settings::setOption( 'fields/custom', Helper::jsonDecode( $customFields, true ) );
                }

                $availableFields = Settings::getOption( 'fields/available' );
                if ( Helper::isNonEmptyStr( $availableFields ) ) {
                        Settings::setOption( 'fields/available', Helper::jsonDecode( $availableFields, true ) );
                }

                self::setFields();
        }
}
