<?php



namespace WpieApp\Modules\Import\Controller;

use WpieApp\Core\Utils\DB;
use WpieApp\Core\Utils\Archive;
use WpieApp\Modules\Import\Helpers\Helper;
use WpieApp\Modules\Import\Controller\Settings;
use WpieApp\Modules\Schedules\Controller\Event;
use WpieApp\Modules\Import\Controller\File\Record;
use WpieApp\Modules\Import\Controller\File\Filter;
use WpieApp\Modules\Import\Controller\File\Setup;
use WpieApp\Modules\Import\Controller\File\Bundle;
use WpieApp\Modules\Import\Controller\File\Uploader\Manager as Uploader;
use WpieApp\Modules\Import\Controller\File\Downloader\Manager as Downloader;
use WpieApp\Modules\Import\Controller\File\Parser\Manager as FileParser;
use WpieApp\Modules\Import\Controller\Items\Manager as Items;
use WpieApp\Modules\Logs\Controller\ManageLog;
use WpieApp\Core\Resource\Resource;

defined( 'WPIE__FILE__' ) || exit;


class Ajax {

        
        public static function uploader() {

                Helper::createDir( WPIE_UPLOAD_IMPORT_DIR );

                if ( !Helper::setWriteble( WPIE_UPLOAD_IMPORT_DIR ) ) {

                        Helper::ajaxError( __( 'Uploads folder is not writable', 'wp-import-export' ) );
                }

                Settings::autoLoad();

                $data = Uploader::init();

                if ( \is_wp_error( $data ) ) {
                        Helper::ajaxError( $data );
                }

                Helper::ajaxSuccess( [ "id" => Settings::get( "id" ), 'data' => $data ] );
        }

        
        public static function downloader() {

                Helper::createDir( WPIE_UPLOAD_IMPORT_DIR );

                if ( !Helper::setWriteble( WPIE_UPLOAD_IMPORT_DIR ) ) {

                        Helper::ajaxError( __( 'Uploads folder is not writable', 'wp-import-export' ) );
                }

                Settings::autoLoad();

                $operation = Helper::sanitizeText( Helper::_post( "operation" ) );

                $options = [
                        'path' => Helper::_post( 'path' ),
                        'url'  => Helper::_post( 'settings/uploder/options/url' ),
                ];

                $data = Downloader::init( $operation, $options );

                if ( \is_wp_error( $data ) ) {
                        Helper::ajaxError( $data );
                }

                Helper::ajaxSuccess( [ "id" => Settings::get( "id" ), 'data' => $data ] );
        }

        
        public static function fileSetup() {

                Settings::autoLoad();

                Helper::ajaxResponse( Setup::init() );
        }

        
        public static function processFile() {

                Settings::autoLoad();

                Helper::ajaxResponse( FileParser::init() );
        }

        
        public static function getArchiveFiles() {

                Settings::autoLoad();

                $files = Archive::getFileList( Settings::getUserFilePath(), [ 'allowedExt' => Helper::getSupportedFileExt() ] );

                Helper::ajaxResponse( \is_wp_error( $files ) ? $files : [ 'files' => $files ] );
        }

        
        public static function extract() {

                Settings::autoLoad();

                $options = [
                        'outputFolder' => Settings::getCurrentDir( "file" )
                ];

                $file = Helper::_post( "file" );

                Settings::setOption( 'archive/file', $file );

                $isSucess = Archive::extract( Settings::getUserFilePath(), $file, $options );

                Helper::ajaxResponse( $isSucess );
        }

        
        public static function getRecord() {

                Settings::autoLoad();

                Helper::ajaxResponse( Record::get( "getRow", [ "row" => Helper::_post( "row" ) ] ) );
        }

        
        public static function getFilterElements() {

                Settings::autoLoad();

                Helper::ajaxResponse( Record::get( "getFilterElements" ) );
        }

        
        public static function getTotal() {

                Settings::autoLoad();

                $total = Record::get( "getTotal" );

                if ( is_numeric( $total ) ) {
                        Settings::set( 'total', $total );
                }

                Helper::ajaxResponse( $total );
        }

        
        public static function updateOptions() {

                Settings::autoLoad();

                Settings::setOptions( Helper::_post( "wpie-settings" ) );

                Helper::ajaxSuccess();
        }

        
        public static function duplicateSettings() {

                Settings::autoLoad();

                Helper::ajaxResponse( Template::duplicate() );
        }

        
        public static function findDuplicateImport() {

                Settings::autoLoad();

                Helper::ajaxSuccess( Settings::findDuplicate() );
        }

        
        public static function duplicateFile() {

                Settings::autoLoad();

                Helper::ajaxSuccess( Template::duplicateFile() );
        }

        
        public static function detectContentType() {

                Settings::autoLoad();

                Helper::ajaxSuccess( ContentType::find() );
        }

        
        public static function getFields() {

                Settings::autoLoad();

                Settings::setContentType( Helper::_post( "content_type" ) );

                Helper::ajaxSuccess( Items::get( 'fields' ) );
        }

        
        public static function getExistingItemsFields() {

                Settings::autoLoad();

                Helper::ajaxSuccess( Items::get( 'ExistingItemsFields' ) );
        }

        
        public static function getAutoLoadFields() {

                Settings::autoLoad();

                Helper::ajaxSuccess( Items::get( 'AutoLoadFields' ) );
        }

        
        public static function updateFilter() {

                Settings::autoLoad();

                Settings::setOption( "data/filters", Helper::_post( "filters" ) );

                Helper::ajaxResponse( Filter::init() );
        }

        
        public static function getFiles() {

                Settings::autoLoad();

                Helper::ajaxResponse( Setup::getFiles() );
        }

        
        public static function setFile() {

                Settings::autoLoad();

                $file = Helper::_post( "path" );

                Helper::ajaxResponse( Setup::setFile( $file ) );
        }

        
        public static function setConfig() {

                Settings::autoLoad();

                Helper::ajaxResponse( Bundle::setConfig() );
        }

        
        public static function getSheets() {

                if ( !\class_exists( '\ZipArchive' ) ) {
                        Helper::ajaxError( __( 'Please enable PHP ZIP extension', 'wp-import-export' ) );
                }

                Settings::autoLoad();

                Helper::ajaxResponse( Setup::getSheets() );
        }

        
        public static function setSheet() {

                Settings::autoLoad();

                $sheet = Helper::_post( "sheet" );

                Helper::ajaxResponse( Setup::setSheet( $sheet ) );
        }

        
        public static function parseFile() {
                Resource::increase();
                Settings::autoLoad();

                $retry = Helper::toInt( Helper::_post( "retry" ) );

                Helper::ajaxResponse( FileParser::init( $retry ) );
        }

        
        public static function getSetting() {

                Settings::autoLoad();

                $data = [
                        "process_type" => Settings::get( 'process_type' ),
                        "content_type" => Settings::get( 'content_type' ),
                        "category"     => Settings::get( 'category' ),
                        "options"      => Settings::get( 'options' ),
                        "total"        => Settings::get( 'total' ),
                        "file"         => Settings::get( 'file' ),
                ];

                Helper::ajaxResponse( $data );
        }

        
        public static function importData() {

                Settings::autoLoad();

                if ( 'processing' !== Settings::get( 'status' ) ) {
                        Settings::set( [ 'status' => 'processing' ] );
                }

                $result = Items::get();

                if ( 'processing' === Settings::get( 'status' ) ) {
                        Settings::set( [ 'status' => 'waiting' ] );
                }
                Helper::ajaxResponse( $result );
        }

        
        public static function getLog() {

                Settings::autoLoad();

                Helper::ajaxResponse( [
                        "processed" => Settings::getLog( "processed" ),
                        "skipped"   => Settings::getLog( "skipped" ),
                        "created"   => Settings::getLog( "created" ),
                        "updated"   => Settings::getLog( "updated" ),
                        "status"    => Settings::get( "status" ) === "completed" ? "completed" : "pending",
                ] );
        }

        
        public static function getNodeList() {

                Settings::autoLoad();

                $nodes = Record::get( 'getNodeList' );

                Helper::ajaxResponse( \is_wp_error( $nodes ) ? $nodes : [ 'nodes' => $nodes, 'roots' => Record::get( 'getRootList', [ "nodes" => $nodes ] ) ] );
        }

        
        public static function updateSettings() {

                $id = Helper::toInt( Helper::_post( "id" ) );

                if ( $id < 1 ) {
                        Helper::ajaxError( __( 'Settings not found', 'wp-import-export' ) );
                }

                Settings::load( $id );

                Settings::set( Helper::_post( "settings" ) );

                Settings::setOptions( Helper::_post( "options" ) );

                Helper::ajaxSuccess();
        }

        
        public static function saveSettings() {

                Settings::autoLoad();

                Settings::setOptions( Helper::_post( "wpie-settings" ) );

                Helper::ajaxResponse( Settings::saveTemplate( Helper::_post( "newId" ) ) );
        }

        
        public static function deleteSettings() {

                Helper::ajaxResponse( ManageLog::removeLogs( [ Helper::_post( "id" ) ] ) );
        }

        
        public static function saveSchedule() {

                Settings::autoLoad();

                Settings::setOptions( Helper::_post( "wpie-settings" ) );

                $name = Settings::getOption( 'schedule/name' );

                if ( !Helper::isNonEmptyStr( $name ) ) {
                        Helper::ajaxError( __( 'Schedule name required', 'wp-import-export' ) );
                }

                $existingId = DB::wpdb()->get_var( DB::wpdb()->prepare( "SELECT id FROM " . DB::getPluginTable() . " where `name` = %s Limit 0,1", $name ) );

                if ( intval( $existingId ) > 0 ) {
                        Helper::ajaxError( __( 'Schedule name already exist', 'wp-import-export' ) );
                }

                Settings::set( [ 'process_type' => 'schedule', 'name' => $name, 'status' => 'completed' ] );

                $id = Settings::get( 'id' );

                $interval = Settings::getOption( 'schedule/interval' );

                $start = Settings::getOption( 'schedule/start' );

                $isAdded = Event::add( $id, $interval, $start );

                Helper::ajaxResponse( is_wp_error( $isAdded ) ? $isAdded : __( 'Schedule Saved Successfully', 'wp-import-export' ) );
        }

        
        public static function getExportSetting() {

                Helper::ajaxResponse( AutoLoader::init() );
        }

        
        public static function downloadLog() {

                Settings::autoLoad();

                Helper::download( Settings::getLogFilePath() );
        }

        
        public static function updateStatus() {

                Settings::autoLoad();

                $importStatus = Settings::get( 'status' );

                if ( in_array( $importStatus, [ 'completed', 'cancelled', 'failed', 'stopped' ] ) ) {
                        /* translators: %s: Import Status */
                        Helper::ajaxError( sprintf( __( 'Import already %s', 'wp-import-export' ), $importStatus ) );
                }

                $status = Helper::_post( "status" );

                $data = ('background' === $status) ? [ 'status' => 'processing', 'process_type' => 'background' ] : [ 'status' => $status ];

                Settings::set( $data );

                Helper::ajaxSuccess();
        }

        
        public static function duplicate() {

                Settings::autoLoad();

                Helper::ajaxResponse( Process::duplicate() );
        }

        
        public static function reImport() {

                Settings::autoLoad();

                $status = 'init';

                if ( 'completed' === Process::init() ) {
                        $status = 'completed';
                        Settings::set( 'status', 'draft' );
                }

                if ( 'failed' === Settings::get( 'status' ) ) {

                        $error = Settings::getOption( 'error' );

                        if ( Helper::isNonEmptyStr( $error ) ) {
                                Helper::ajaxError( $error );
                        } else {
                                $status = 'failed';
                        }
                }

                Helper::ajaxResponse( $status );
        }
}
