<?php



namespace WpieApp\Modules\Import\Controller\File\Downloader\Url\Downloader\FileNameFinder;

use WP_Error;
use GuzzleHttp\Psr7\Utils;
use WpieApp\Modules\Import\Helpers\Helper;
use WpieApp\Modules\Import\Controller\File\Downloader\Url\Downloader\Headers\Manager as Headers;

defined( 'WPIE__FILE__' ) || exit;


class Manager {

        
        Private static $uri;

        
        Private static $headers = [];

        
        Private static $mimeTypes = [];

        
        public static function find( $uri ) {

                self::$uri = Utils::uriFor( $uri );

                $headers = Headers::get( self::$uri );

                if ( is_wp_error( $headers ) ) {
                        return $headers;
                }

                self::$headers = $headers;

                return self::getName();
        }

        
        private static function getExtension() {

                $mimeType = isset( self::$headers[ 'content-type' ] ) ? self::$headers[ 'content-type' ] : "";

                if ( Helper::isNonEmptyArray( $mimeType ) ) {
                        $mimeType = end( $mimeType );
                }

                if ( !Helper::isNonEmptyStr( $mimeType ) ) {
                        return "";
                }

                $mimeTypes = Helper::strToArr( $mimeType, ";" );

                $fileExt = "";

                foreach ( $mimeTypes as $type ) {

                        if ( !Helper::isNonEmptyStr( $type ) ) {
                                continue;
                        }

                        $extension = self::getExtensionForMimeType( trim( $type ) );

                        if ( !Helper::isNonEmptyStr( $extension ) ) {
                                continue;
                        }
                        $fileExt = $extension;
                }

                if ( Helper::isNonEmptyStr( $fileExt ) ) {
                        return $fileExt;
                }

                return "";
        }

        
        private static function getName() {
                $names = [
                        self::getNameByUrl(),
                        self::getNameByHeaders()
                ];

                $fileExt = self::getExtension();

                $fileName = self::getValidName( $names, $fileExt );

                if ( Helper::isNonEmptyStr( $fileName ) ) {
                        return $fileName;
                }

                return new WP_Error( 'wpieError', __( "File Not Allowed", 'wp-import-export' ) );
        }

        
        private static function getValidName( $names = [], $ext = "" ) {


                if ( !Helper::isNonEmptyArray( $names ) ) {
                        return self::getRandName( $ext );
                }

                $file = "";

                foreach ( $names as $name ) {

                        if ( !Helper::isNonEmptyStr( $name ) || !self::isFileName( $name ) ) {
                                continue;
                        }

                        $fileExt = Helper::getFileExt( $name );

                        if ( Helper::isNonEmptyStr( $ext ) && $fileExt !== $ext ) {
                                continue;
                        }

                        $file = $name;

                        break;
                }

                if ( Helper::isNonEmptyStr( $file ) ) {
                        return $file;
                }

                return self::getRandName( $ext );
        }

        
        private static function getRandName( $ext = "" ) {

                if ( !Helper::isNonEmptyStr( $ext ) ) {
                        return "";
                }

                return Helper::randStr() . "." . $ext;
        }

        
        private static function isFileName( $name ) {

                if ( !Helper::isNonEmptyStr( $name ) ) {
                        return false;
                }

                $file = Helper::strToArr( $name, "." );

                if ( count( $file ) < 2 ) {
                        return false;
                }

                $fileInfo = \wp_check_filetype( $name, self::getMimeTypes() );

                if ( !isset( $fileInfo[ 'ext' ] ) || !Helper::isNonEmptyStr( $fileInfo[ 'ext' ] ) ) {
                        return false;
                }

                return true;
        }

        
        private static function getNameByUrl() {

                $path = self::$uri->getPath();

                return Helper::sanitizeFilename( basename( $path ) );
        }

        
        private static function getNameByHeaders() {

                $header = isset( self::$headers[ 'content-disposition' ] ) ? self::$headers[ 'content-disposition' ] : "";

                if ( !Helper::isNonEmptyStr( $header ) ) {
                        return false;
                }
                if ( !Helper::contains( $header, 'filename' ) ) {
                        return false;
                }

                $data = [];

                $regex = '/.*?filename=(?<fn>[^\s]+|\x22[^\x22]+\x22)\x3B?.*$/m';

                if ( preg_match( $regex, $header, $data ) === 1 ) {

                        if ( isset( $data[ 'fn' ] ) && Helper::isNonEmptyStr( $data[ 'fn' ] ) ) {
                                return Helper::sanitizeFilename( $data[ 'fn' ] );
                        }
                }

                $data = [];

                $regex = '/.*filename=([\'\"]?)([^\"]+)\1/';

                if ( preg_match( $regex, $header, $data ) === 1 ) {

                        if ( isset( $data[ 2 ] ) && Helper::isNonEmptyStr( $data[ 2 ] ) ) {
                                return Helper::sanitizeFilename( $data[ 2 ] );
                        }
                }

                return false;
        }

        
        private static function getMimeTypes() {

                if ( !Helper::isNonEmptyArray( self::$mimeTypes ) ) {

                        self::$mimeTypes           = \get_allowed_mime_types();
                        self::$mimeTypes[ 'xml' ]  = "text/xml";
                        self::$mimeTypes[ 'svg' ]  = "image/svg+xml";
                        self::$mimeTypes[ 'json' ] = "application/json";
                }

                return self::$mimeTypes;
        }

        
        private static function getAdvancedMimeTypes() {

                return [
                        'application/vnd.api+json' => "json",
                        'application/xml'          => 'xml',
                        'application/rss+xml'      => 'xml'
                ];
        }

        
        private static function getExtensionForMimeType( $mime_type = "" ) {

                $extensions = array_search( $mime_type, self::getMimeTypes(), true );

                if ( $extensions !== false ) {
                        $extensions = Helper::strToArr( $extensions, '|' );
                        return isset( $extensions[ 0 ] ) ? $extensions[ 0 ] : "";
                }

                $advancedMimes = self::getAdvancedMimeTypes();

                if ( isset( $advancedMimes[ $mime_type ] ) ) {
                        return $advancedMimes[ $mime_type ];
                }

                return false;
        }
}
