<?php



namespace WpieApp\Modules\Import\Controller\File\Filters;

use WpieApp\Modules\Import\Helpers\Helper;
use WpieApp\Modules\Import\Controller\File\Records\Csv as CsvReader;
use WpieApp\Core\Office\Writer\CSV\Writer as CsvWriter;
use WpieApp\Modules\Import\Controller\Settings;
use WpieApp\Core\Resource\Resource;

defined( 'WPIE__FILE__' ) || exit;


class Csv extends Base {

        
        protected function preInit() {
                
        }

        
        protected function initReader() {
                return CsvReader::getReader( false );
        }

        
        protected function initWriter() {

                $options = [
                        'delimiter' => Settings::getOption( "file/delimiter", null ),
                        'enclosure' => Settings::getOption( "file/enclosure" )
                ];

                $file = Settings::getFilteredFilePath();

                $this->writer = new CsvWriter();

                return $this->writer->load( $file, $options );
        }

        
        protected function applyFilters() {

                $this->addLabels();

                $isExceeded = false;

                while ( $this->reader->next() ) {

                        if ( Resource::isExceeded() ) {
                                $isExceeded = true;
                                break;
                        }

                        $this->data = $this->reader->current();

                        if ( !Helper::isNonEmptyArray( $this->data ) ) {
                                continue;
                        }

                        if ( !$this->applyRules() ) {
                                continue;
                        }

                        $this->writer->addRow( array_values( $this->data ) );
                }


                $this->writer->clear();

                if ( $isExceeded ) {

                        Settings::setOptions( [
                                "filter/status"  => "pending",
                                "filter/pointer" => ( string ) $this->reader->getPointer(),
                                "filter/row"     => ( string ) $this->reader->getRowNumber(),
                        ] );

                        return [ "status" => "processing" ];
                }

                Settings::setOption( "filter/status", "completed" );

                return [ "status" => "completed" ];
        }

        
        protected function resumeTask() {

                if ( Settings::getOption( "filter/status" ) !== "pending" ) {
                        return;
                }

                $this->rules = Settings::getOption( "filter/rules" );

                $row = Settings::getOption( "filter/row" );

                $pointer = Settings::getOption( "filter/pointer" );

                $this->reader->setPointer( $pointer, $row );
        }

        
        private function addLabels() {

                if ( Settings::getOption( "filter/status" ) === "pending" ) {
                        return;
                }

                $hasTitle = Settings::getOption( "file/hasTitle" );

                if ( Helper::toInt( $hasTitle ) !== 1 && '' !== $hasTitle ) {
                        return;
                }

                $labels = $this->reader->getLabels();

                if ( !Helper::isNonEmptyArray( $labels ) ) {
                        return;
                }

                $this->writer->addRow( $labels );
        }
}
