<?php



namespace WpieApp\Modules\Import\Controller\Items\Base;

use WpieApp\Core\Resource\Resource;
use WpieApp\Modules\Import\Helpers\Helper;
use WpieApp\Modules\Import\Controller\Settings;

defined( 'WPIE__FILE__' ) || exit;


class Engine implements EngineInterface {

        
        public static function import() {
                
        }

        
        public static function searchDuplicates() {
                
        }

        
        public static function init() {

                Logger::init();

                Process::init();

                $isInit = DataCenter::init();

                if ( \is_wp_error( $isInit ) ) {
                        return $isInit;
                }

                static::start();

                return [
                        "processed" => Settings::getLog( "processed" ),
                        "skipped"   => Settings::getLog( "skipped" ),
                        "created"   => Settings::getLog( "created" ),
                        "updated"   => Settings::getLog( "updated" ),
                        "status"    => Settings::get( "status" ) === "completed" ? "completed" : "pending",
                ];
        }

        
        public static function start() {

                /* translators: %s: Content type */
                Logger::log( sprintf( __( "Start %s Importing", 'wp-import-export' ), static::getContentTypeLabel() ) );

                if ( Settings::getLog( "processed" ) === 0 ) {

                        \do_action( "Wpie/Import/Items/Start" );
                }

                static::preStart();

                static::processing();
        }

        
        public static function complete() {

                \do_action( "Wpie/Import/Items/End" );

                /* translators: %s: Content type */
                Logger::log( sprintf( __( "End %s Importing", 'wp-import-export' ), static::getContentTypeLabel() ) );

                Settings::set( "status", "completed" );
        }

        
        public static function next() {

                if ( Resource::isExceeded( 85 ) ) {
                        Logger::log( __( "Resource is exceeded", 'wp-import-export' ) );
                        return false;
                }

                $record = Settings::getLog( "processed" ) + 1;

                if ( $record > Settings::get( "total" ) || !DataCenter::next() ) {
                        static::complete();
                        return false;
                }

                return true;
        }

        
        protected static function canAddUpdate() {

                $handleItems = Settings::getOption( "handle-items" );

                if ( !Helper::isNonEmptyArray( $handleItems ) ) {
                        return true;
                }

                if ( !DataCenter::isNewItem() && !in_array( "update", $handleItems ) ) {
                        /* translators: %s: Content type */
                        Logger::log( "<strong>" . __( "SKIPPED", 'wp-import-export' ) . "</strong> " . sprintf( __( "duplicate %s.", 'wp-import-export' ), static::getContentTypeLabel() ) );

                        return false;
                } else if ( DataCenter::isNewItem() && !in_array( "add", $handleItems ) ) {
                        /* translators: %s: Content type */
                        Logger::log( "<strong>" . __( "SKIPPED", 'wp-import-export' ) . "</strong> " . sprintf( __( "new %s.", 'wp-import-export' ), static::getContentTypeLabel() ) );

                        return false;
                }

                unset( $handleItems );

                return true;
        }

        
        protected static function baseSearch() {

                \do_action( "Wpie/Import/Items/Engine/Search/Init" );

                $searchId = \apply_filters( "Wpie/Import/Items/Engine/Search/Start", Process::getItemId() );

                $id = (is_int( $searchId ) && $searchId > 0) ? $searchId : static::searchDuplicates();

                $id = \apply_filters( "Wpie/Import/Items/Engine/Search/End", $id );

                if ( $id > 0 ) {
                        /* translators: 1: Content type. 2: Item id */
                        Logger::log( sprintf( __( 'Duplicate %1$s #%2$s Found', 'wp-import-export' ), static::getContentTypeLabel(), $id ) );

                        DataCenter::setId( $id );
                }

                DataCenter::setNewItem( $id > 0 ? false : true );
        }

        
        protected static function baseImport() {

                $successImport = false;

                if ( static::canAddUpdate() ) {

                        \do_action( "Wpie/Import/Base/Engine/Start" );

                        $successImport = static::import();

                        \do_action( "Wpie/Import/Base/Engine/End", $successImport );
                }

                static::updateLogs( $successImport );
        }

        
        protected static function processing() {

                static::startRecord();

                static::preProcessing();

                static::updateEngine();

                static::baseSearch();

                static::baseImport();

                static::endRecord();

                if ( Process::isPending() || !static::next() ) {

                        return true;
                }

                static::processing();
        }

        
        private static function startRecord() {

                Process::reset();

                $record = Settings::getLog( "processed" ) + 1;

                \do_action( "Wpie/Import/Items/Record/Start", $record );

                /* translators: %d: Record number */
                Logger::log( sprintf( __( "<strong>Record #%d</strong> : Start Importing", 'wp-import-export' ), $record ) );
        }

        
        private static function updateLogs( $successImport ) {

                if ( !$successImport ) {
                        Settings::updateLog( "skipped" );
                        return;
                }

                if ( DataCenter::isNewItem() ) {
                        /* translators: 1: content type. 2: item id */
                        Logger::log( "<strong>" . __( "CREATED", 'wp-import-export' ) . "</strong> " . sprintf( __( '%1$s ID #%2$d successfully.', 'wp-import-export' ), static::getContentTypeLabel(), DataCenter::getId() ) );
                        Process::isPending() ? '' : Settings::updateLog( "created" );
                } else {
                        /* translators: 1: content type. 2: item id */
                        Logger::log( "<strong>" . __( "UPDATED", 'wp-import-export' ) . "</strong> " . sprintf( __( '%1$s ID #%2$d successfully.', 'wp-import-export' ), static::getContentTypeLabel(), DataCenter::getId() ) );
                        Process::isPending() ? '' : Settings::updateLog( "updated" );
                }

                if ( Process::isPending() ) {
                        /* translators: 1: content type. 2: item id */
                        Logger::log( "<strong>" . __( "PENDING", 'wp-import-export' ) . "</strong> " . sprintf( __( '%1$s ID #%2$d Importing is still pending.', 'wp-import-export' ), static::getContentTypeLabel(), DataCenter::getId() ) );
                }
                
                DataCenter::updateLogs();

                Process::save();
        }

        
        protected static function preStart() {
                
        }

        
        protected static function preProcessing() {
                \do_action( "Wpie/Import/Base/Engine/preProcessing" );
        }

        
        protected static function updateEngine() {
                
        }

        
        private static function endRecord() {

                $record = Settings::getLog( "processed" );

                \do_action( "Wpie/Import/Items/Record/End", $record );

                /* translators: %d: record number */
                Logger::log( sprintf( __( "<strong>Record #%d</strong> : End Importing", 'wp-import-export' ), $record ) );

                DataCenter::clear();
        }

        
        public static function getContentTypeLabel() {

                return Settings::get( 'content_type' );
        }

        
        protected static function afterAdd() {

                \do_action( "Wpie/Import/Item/Engine/Add/After" );
        }
}
