<?php



namespace WpieApp\Modules\Import\Controller\Items\Base\Engines\Images;

use WpieApp\Core\Utils\Date;
use WpieApp\Core\Resource\Resource;
use WpieApp\Modules\Import\Helpers\Helper;
use WpieApp\Modules\Import\Controller\Settings;
use WpieApp\Modules\Import\Controller\Items\Base\DataCenter;
use WpieApp\Modules\Import\Controller\Items\Base\Logger;
use WpieApp\Modules\Import\Controller\Items\Base\Process;
use WpieApp\Modules\Import\Controller\File\Downloader\Url\Downloader\File;

defined( 'WPIE__FILE__' ) || exit;


class Source {

        
        protected static $data = [];

        
        protected static $attachments = [];

        
        protected static $imageNames = [];

        
        protected static $allowedExtensions = [];

        
        protected static $tempDirectory = "";

        
        public static function import() {

                if ( !DataCenter::canUpdate( "images/source" ) ) {
                        return false;
                }

                static::setExtensions();

                static::init();

                static::setData();

                static::processImages();

                static::setImages();

                return true;
        }

        
        private static function init() {

                static::$data = [];

                static::$attachments = [];

                static::$tempDirectory = Helper::isNonEmptyArray( static::$tempDirectory ) ? static::$tempDirectory : \apply_filters( 'Wpie/Import/Images/Upload/Directory', WPIE_UPLOAD_TEMP_DIR );
        }

        
        private static function setExtensions() {

                if ( Helper::isNonEmptyArray( static::$allowedExtensions ) ) {
                        return;
                }

                static::$allowedExtensions = Helper::getImageExtensions();
        }

        
        private static function setData() {

                $imageSource = DataCenter::get( "images/source/data" );

                if ( !Helper::isNonEmptyStr( $imageSource ) ) {
                        return;
                }

                $images = Helper::strToArr( $imageSource, "\n" );

                if ( count( $images ) <= 1 ) {

                        $separator = DataCenter::get( "images/source/separator" );

                        $images = Helper::strToArr( $imageSource, $separator );
                }

                static::$data = $images;
        }

        
        protected static function getDate() {

                return \time();
        }

        
        private static function setNewNames() {

                if ( !DataCenter::canUpdate( "images/rename" ) ) {
                        return false;
                }

                $imageNames = DataCenter::get( "images/rename/data" );

                if ( !Helper::isNonEmptyStr( $imageNames ) ) {
                        return;
                }

                $names = Helper::strToArr( $imageNames, "\n" );

                if ( count( $names ) <= 1 ) {

                        $separator = DataCenter::get( "images/rename/separator" );

                        $separator = Helper::isNonEmptyStr( $separator ) ? $separator : ',';

                        $names = Helper::strToArr( $imageNames, $separator );
                }

                //If names list are not same with source, none of apply
                if ( count( static::$data ) !== count( $names ) ) {
                        return;
                }

                static::$imageNames = $names;
        }

        
        private static function processImages() {

                if ( !Helper::isNonEmptyArray( static::$data ) ) {
                        return;
                }

                static::setNewNames();

                foreach ( static::$data as $key => $image ) {

                        if ( !Helper::isNonEmptyStr( $image ) ) {
                                continue;
                        }

                        if ( Resource::isExceeded( 80 ) ) {
                                Process::setPending();
                                Logger::log( __( "Resource is exceeded", 'wp-import-export' ) );
                                Logger::log( __( "Image import is currently in the middle of processing", 'wp-import-export' ) );
                                break;
                        }

                        $image = trim( $image );

                        $attch = static::search( $image );

                        if ( is_numeric( $attch ) && absint( $attch ) > 0 ) {
                                static::$attachments[] = $attch;
                                continue;
                        }

                        $newName = self::getNewName( $image, $key );

                        if ( Helper::isNonEmptyStr( $newName ) ) {

                                $attch = static::search( $newName );

                                if ( is_numeric( $attch ) && absint( $attch ) > 0 ) {
                                        static::$attachments[] = $attch;
                                        continue;
                                }
                        }

                        $attch = static::add( $image, $newName );

                        if ( is_numeric( $attch ) && absint( $attch ) > 0 ) {
                                static::$attachments[] = $attch;
                                continue;
                        } elseif ( is_wp_error( $attch ) ) {
                                Logger::log( $attch->get_error_message() );
                                continue;
                        } elseif ( absint( $attch ) < 1 ) {
                                /* translators: %s: Image name */
                                Logger::log( sprintf( __( "Image %s not found", 'wp-import-export' ), $image ) );
                                continue;
                        }
                }
        }

        
        private static function add( $image, $newName ) {

                $imageOption = DataCenter::get( "images/source/option" );

                $file = null;

                if ( $imageOption === "url" ) {

                        $file = static::DownloadImage( $image );
                } elseif ( $imageOption === "dir" ) {
                        $file = static::useDirectoryImage( $image );
                }

                if ( is_wp_error( $file ) || !Helper::isNonEmptyStr( $file ) || !file_exists( $file ) ) {
                        return $file;
                }

                if ( Helper::isNonEmptyStr( $newName ) && basename( $file ) !== basename( $newName ) && Helper::getFilesystem()->move( $file, dirname( $file ) . '/' . basename( $newName ), true ) ) {
                        $file = dirname( $file ) . '/' . basename( $newName );
                }

                return static::insertAttachment( $file, $image, $imageOption );
        }

        
        public static function upload( $image ) {

                static::setExtensions();

                $file = static::DownloadImage( $image );

                if ( \is_wp_error( $file ) || !Helper::isNonEmptyStr( $file ) || !file_exists( $file ) ) {
                        return $file;
                }

                return static::insertAttachment( $file, $image );
        }

        
        private static function useDirectoryImage( $image = "" ) {
                return Helper::pathJoin( static::$tempDirectory, $image );
        }

        
        private static function DownloadImage( $image = "" ) {

                $uri = new File( $image, static::$allowedExtensions );

                $filename = $uri->getFilename();

                if ( basename( $image ) !== $filename ) {

                        $id = static::search( $filename );

                        if ( is_numeric( $id ) && absint( $id ) > 0 ) {
                                unset( $uri );
                                return \get_attached_file( $id );
                        }
                }

                $attach = $uri->download();

                unset( $uri );

                if ( \is_wp_error( $attach ) ) {
                        return $attach;
                }

                $path = isset( $attach[ 'path' ] ) ? $attach[ 'path' ] : "";

                if ( Helper::getFilesystem()->exists( $path ) ) {
                        $newPath = dirname( $path ) . '/' . basename( $filename );
                        Helper::getFilesystem()->move( $path, $newPath, true );
                        $path    = $newPath;
                }

                return $path;
        }

        
        public static function search( $image = "" ) {
                return 0;
        }

        
        protected static function insertAttachment( $file = "", $source = "", $sourceMethod = "url" ) {

                if ( !file_exists( $file ) ) {
                        return false;
                }

                $uploadFile = static::uploadFile( $file, $source );

                if ( $uploadFile === false ) {
                        return false;
                }

                $fileType = \wp_check_filetype( $file );

                $defaults = static::getDefaults();

                $options = array_replace( $defaults,
                        [
                                'post_title'     => basename( $file ),
                                'post_type'      => 'attachment',
                                'post_status'    => 'inherit',
                                'post_mime_type' => isset( $fileType[ 'type' ] ) ? $fileType[ 'type' ] : "",
                        ]
                );

                $id = \wp_insert_attachment( $options, $uploadFile, DataCenter::getId() );

                if ( is_int( $id ) && $id > 0 ) {

                        // The image sub-sizes are created during wp_generate_attachment_metadata().
                        // This is generally slow and may cause timeouts or out of memory errors.
                        \wp_update_attachment_metadata( $id, \wp_generate_attachment_metadata( $id, $uploadFile ) );

                        if ( $sourceMethod === "url" ) {
                                \update_post_meta( $id, "_wpie_source_url", $source );
                        }
                }

                return $id;
        }

        
        protected static function getDefaults() {
                return [];
        }

        
        private static function uploadFile( $file = "", $source = "" ) {

                $uploads = \wp_upload_dir( static::getMediaTime( $source ) );

                $name = \wp_unique_filename( $uploads[ 'path' ], basename( $file ) );

                // Move the file to the uploads dir.
                $newFile = $uploads[ 'path' ] . "/" . $name;

                if ( is_dir( $file ) || !Helper::duplicate( $file, $newFile, false ) ) {
                        /* translators: %s: file path */
                        Logger::log( sprintf( __( "Can't copy %s File", 'wp-import-export' ), $file ) );
                        return false;
                }

                Helper::deleteFile( $file );

                return $newFile;
        }

        
        private static function getMediaTime( $url = "" ) {

                $subPath = "/wp-content/uploads";

                if ( strpos( $url, $subPath ) !== false ) {
                        return mb_substr( $url, strpos( $url, $subPath ) + mb_strlen( $subPath ) + 1, 7 );
                }

                return Date::get( static::getDate(), "Y/m" );
        }

        
        public static function getAttachments() {

                return static::$attachments;
        }

        
        public static function setImages() {
                
        }

        
        public static function getNewName( $image, $key ) {

                if ( !isset( static::$imageNames[ $key ] ) || !Helper::isNonEmptyStr( static::$imageNames[ $key ] ) ) {
                        return '';
                }

                $newName = static::$imageNames[ $key ];

                $ext = Helper::getFileExt( $newName );

                $ext = Helper::isNonEmptyStr( $ext ) && in_array( $ext, self::$allowedExtensions ) && $newName !== $ext ? $ext : Helper::getFileExt( basename( $image ) );

                return Helper::sanitizeFilename( $newName, $ext );
        }
}
