<?php



namespace WpieApp\Modules\Import\Controller\Items\Term;

use WpieApp\Modules\Import\Helpers\Helper;
use WpieApp\Modules\Import\Controller\Items\Term\Settings;
use WpieApp\Modules\Import\Controller\Items\Term\Search\Term;
use WpieApp\Modules\Import\Controller\Items\Term\AutoSearch\Data;

defined( 'WPIE__FILE__' ) || exit;


class AutoSearch {

        
        public static function get() {

                $term = self::tryAll();

                if ( is_numeric( $term ) && 0 < absint( $term ) ) {
                        return $term;
                }

                $term = self::trySlug();

                if ( is_numeric( $term ) && 0 < absint( $term ) ) {
                        return $term;
                }

                $term = self::tryName();

                if ( is_numeric( $term ) && 0 < absint( $term ) ) {
                        return $term;
                }

                return 0;
        }

        
        public static function tryAll() {

                $slug = Data::getSlug();

                $name = Data::getName();

                if ( false === $slug || 'auto' === $slug || !Helper::isNonEmptyStr( $slug ) || false === $name || !Helper::isNonEmptyStr( $name ) ) {
                        return false;
                }

                $arg = [ 'name' => $name, 'slug' => $slug ];

                $terms = self::runQuery( $arg );

                if ( false === $terms || 0 === $terms ) {
                        return $terms;
                }

                return array_shift( $terms );
        }

        
        public static function trySlug() {

                $slug = Data::getSlug();

                if ( false === $slug || !Helper::isNonEmptyStr( $slug ) ) {
                        return false;
                }

                $terms = self::runQuery( [ 'slug' => $slug ] );

                if ( false === $terms || 0 === $terms ) {
                        return $terms;
                }

                return array_shift( $terms );
        }

        
        public static function tryName() {

                $name = Data::getName();

                if ( false === $name || !Helper::isNonEmptyStr( $name ) ) {
                        return false;
                }

                $terms = self::runQuery( [ 'name' => $name ] );

                if ( false === $terms || 0 === $terms ) {
                        return $terms;
                }

                $detectedTerm = self::matchParent( $terms );

                if ( is_numeric( $detectedTerm ) && 0 < $detectedTerm ) {
                        return $detectedTerm;
                }

                $slug = Data::getSlug();

                return Helper::isNonEmptyStr( $slug ) ? 0 : array_shift( $terms );
        }

        
        public static function matchParent( $includeTerms = [] ) {

                $taxonomy = Settings::getTaxonomy();

                if ( !\is_taxonomy_hierarchical( $taxonomy ) ) {
                        return false;
                }

                $parent = Data::getParent();

                if ( !is_numeric( $parent ) || 1 > $parent ) {
                        return false;
                }

                $terms = self::runQuery( [ 'include' => $includeTerms, 'parent' => $parent ] );

                if ( false === $terms || 0 === $terms ) {
                        return $terms;
                }

                return 0 < count( $terms ) ? array_shift( $terms ) : 0;
        }

        
        public static function runQuery( $arg = [] ) {

                $taxonomy = Settings::getTaxonomy();

                $defaults = [
                        'taxonomy' => $taxonomy,
                ];

                $arg = array_replace( $arg, $defaults );

                $terms = Search::getTerms( $arg );

                if ( false === $terms || is_wp_error( $terms ) ) {
                        return false;
                }

                return (1 > count( $terms ) ? 0 : $terms);
        }

}
