<?php



namespace WpieApp\Modules\Import\Controller\Items\User;

use WpieApp\Modules\Import\Helpers\Helper;
use WpieApp\Modules\Import\Controller\Settings;
use WpieApp\Modules\Import\Controller\Items\Base\Logger;
use WpieApp\Modules\Import\Controller\Items\Base\Engine as BaseEngine;
use WpieApp\Modules\Import\Controller\Items\User\Engines\Standard;
use WpieApp\Modules\Import\Controller\Items\User\Engines\Notification;
use WpieApp\Modules\Import\Controller\Items\User\Engines\Other;
use WpieApp\Modules\Import\Controller\Items\User\Engines\Meta;
use WpieApp\Modules\Import\Controller\Items\User\DataCenter;

defined( 'WPIE__FILE__' ) || exit;


class Engine extends BaseEngine {

        
        private static $currentUserId = 0;

        
        protected static function preStart() {
                Notification::update();
        }

        
        public static function import() {

                \do_action( "Wpie/Import/User/Engine/Start" );

                if ( !static::insertUser() ) {
                        return false;
                }

                static::afterAdd();

                static::updateUser();

                \do_action( "Wpie/Import/User/Engine/End" );

                return true;
        }

        
        protected static function afterAdd() {

                \do_action( "Wpie/Import/User/Engine/Add/After" );

                parent::afterAdd();
        }

        
        protected static function prepareUser() {

                $standard = Standard::get();

                $others = Other::get();

                return array_merge( $standard, $others );
        }

        
        protected static function setItem( $id = 0 ) {

                DataCenter::setId( $id );

                DataCenter::setItem( \get_user_by( 'id', $id ) );
        }

        
        protected static function updateUser() {

                Standard::update();

                static::updateMeta();
        }

        
        protected static function updateMeta() {

                Meta::import();
        }

        
        protected static function insertUser() {

                $data = \apply_filters( "Wpie/Import/User/Import/Before", static::prepareUser(), Settings::getContentType() );

                $id = ( DataCenter::isNewItem() ? self::insert( $data ) : self::update( $data ));

                if ( false === $id ) {
                        return false;
                }

                if ( \is_wp_error( $id ) ) {

                        Logger::log( "<strong>" . __( "ERROR", 'wp-import-export' ) . "</strong> : " . $id->get_error_message() );

                        return false;
                }

                if ( \absint( $id ) === 0 ) {

                        Logger::log( "<strong>" . __( "ERROR", 'wp-import-export' ) . "</strong> : " . __( "something wrong, ID = 0 was generated.", 'wp-import-export' ) );

                        return false;
                }

                static::setItem( $id );

                return true;
        }

        
        protected static function insert( $data = [] ) {

                if ( !isset( $data[ 'user_login' ] ) ) {
                        Logger::log( "<strong>" . __( "ERROR", 'wp-import-export' ) . "</strong> : " . __( "Cannot create a user with an empty login name.", 'wp-import-export' ) );
                        return false;
                }
                if ( !isset( $data[ 'user_pass' ] ) ) {
                        Logger::log( "<strong>" . __( "ERROR", 'wp-import-export' ) . "</strong> : " . __( "Cannot create a user with an empty password", 'wp-import-export' ) );
                        return false;
                }
                if ( !isset( $data[ 'user_email' ] ) ) {
                        Logger::log( "<strong>" . __( "ERROR", 'wp-import-export' ) . "</strong> : " . __( "Cannot create a user with an empty email", 'wp-import-export' ) );
                        return false;
                }

                if ( isset( $data[ 'ID' ] ) ) {
                        unset( $data[ 'ID' ] );
                }

                return \wp_insert_user( $data );
        }

        
        protected static function update( $data = [] ) {

                if ( !Helper::isNonEmptyArray( $data ) ) {
                        return DataCenter::getId();
                }

                if ( self::getCurrentUserId() === DataCenter::getId() ) {

                        Logger::log( "<strong>" . __( "ERROR", 'wp-import-export' ) . "</strong> : " . __( 'Updating current user data is not allowed', 'wp-import-export' ) );

                        return false;
                }

                $data[ 'ID' ] = DataCenter::getId();

                $result = \wp_update_user( $data );

                if ( \is_wp_error( $result ) ) {

                        Logger::log( "<strong>" . __( "ERROR", 'wp-import-export' ) . "</strong> : " . $result->get_error_message() );
                }

                return DataCenter::getId();
        }

        
        public static function searchDuplicates() {

                $searchMethod = DataCenter::get( "search/method" );

                $id = 0;

                switch ( $searchMethod ) {

                        case "login":
                        case "email":
                                $field = DataCenter::get( "search/" . $searchMethod );

                                $field = Helper::isNonEmptyStr( $field ) ? $field : DataCenter::get( 'standard/' . $searchMethod . '/data' );

                                $id = Helper::isNonEmptyStr( $field ) ? Search::getUserIdBy( $searchMethod, $field ) : 0;

                                break;
                        case "id":
                                $field = DataCenter::get( "search/id" );

                                $id = Helper::isNonEmptyStr( $field ) ? Search::getUserIdBy( 'id', $field ) : 0;

                                break;
                        case "meta":
                                $metaKey   = DataCenter::get( "search/meta/key" );
                                $metaValue = DataCenter::get( "search/meta/value" );

                                if ( Helper::isNonEmptyStr( $metaKey ) && Helper::isNonEmptyStr( $metaValue ) ) {
                                        $id = Search::getUserIdByMeta( $metaKey, $metaValue );
                                }
                                break;

                        default:
                                $id = AutoSearch::get();
                                break;
                }

                return $id;
        }

        
        public static function getCurrentUserId() {

                if ( self::$currentUserId === 0 ) {
                        self::$currentUserId = \get_current_user_id();
                }

                return self::$currentUserId;
        }
}
