<?php



namespace WpieApp\Modules\Import\Helpers;

use WpieApp\Core\Utils\Archive;

defined( 'WPIE__FILE__' ) || exit;


trait FileExt {

        
        private static $fileExt = [ 'txt', 'json', 'xml', 'xls', 'xlsx', 'csv', 'ods' ];

        
        private static $imageExtensions = [];

        
        private static $baseCompressExt = [
                // archived and compressed
                'zip', '7z', 'rar', 'cab', 'tar', 'tgz', 'tbz2', 'txz', 'tar.z', 'arj',
                // compressed
                'gz', 'bz2', 'xz',
                // non-usual archives
                'uefi', 'gpt', 'mbr', 'msi', 'iso', 'dmg', 'udf', 'rpm', 'deb'
        ];

        
        private static $compressExt = [];

        
        public static function getSupportedExt() {

                
                return \apply_filters( 'Wpie/Import/SupportedExt/All', array_merge( self::getSupportedFileExt(), self::getSupportedCompressExt() ) );
        }

        
        public static function getSupportedFileExt() {


                
                return \apply_filters( 'Wpie/Import/SupportedExt/File', self::$fileExt );
        }

        
        public static function getSupportedCompressExt() {

                
                return \apply_filters( 'Wpie/Import/SupportedExt/Compress', self::$baseCompressExt );
        }

        
        public static function setSupportedCompressExt() {

                if ( Helper::isNonEmptyArray( self::$compressExt ) ) {
                        return;
                }
                $compressExt = \array_filter( self::$baseCompressExt, [ Archive::class, 'canOpen' ] );

                self::$compressExt = Helper::isNonEmptyArray( $compressExt ) ? array_values( $compressExt ) : [];

                unset( $compressExt );
        }

        
        public static function isSupportedCompressExt( $ext = "" ) {

                if ( !Helper::isNonEmptyStr( $ext ) || !in_array( $ext, self::$baseCompressExt ) ) {

                        return false;
                }

                if ( Helper::isNonEmptyArray( self::$compressExt ) ) {
                        return in_array( $ext, self::$compressExt );
                }

                return Archive::canOpen( $ext );
        }

        
        public static function isCompressedFile( $file = "" ) {

                return self::isArchive( $file );
        }

        
        public static function isArchive( $file = "" ) {

                if ( !Helper::isNonEmptyStr( $file ) ) {
                        return false;
                }

                $ext = Helper::getFileExt( $file );

                return in_array( $ext, self::$baseCompressExt );
        }

        
        public static function isSupportedFile( $file = "" ) {

                if ( !Helper::isNonEmptyStr( $file ) ) {
                        return false;
                }

                $ext = Helper::getFileExt( $file );

                $fileExt = self::getSupportedFileExt();

                if ( in_array( $ext, $fileExt ) ) {

                        return true;
                }

                return self::isSupportedCompressExt( $ext );
        }

        
        public static function getImageExtensions() {

                if ( Helper::isNonEmptyArray( self::$imageExtensions ) ) {
                        return self::$imageExtensions;
                }

                $allowedMimeTypes = \get_allowed_mime_types();

                $imageExts = [];

                foreach ( $allowedMimeTypes as $mimeType => $extensions ) {
                        if ( strpos( $extensions, 'image/' ) === 0 ) {
                                $imageExts = array_merge( $imageExts, explode( '|', $mimeType ) );
                        }
                }

                
                self::$imageExtensions = \apply_filters( 'Wpie/Import/SupportedExt/Images', array_unique( $imageExts ) );

                return self::$imageExtensions;
        }
}
